/******************************************************************************/
/* Project:   blind control set                                               */
/* Author:    BBC-Hoerer                                                      */
/*            You may use or modify this code for your purpose.               */
/*            BBC-Hoerer is not responsible for consequences using this code. */
/******************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "i2cmaster.h"
#include "lcd.h"
#include "_rl_com.h"

/* init function of alarm manger */
void  alm_manager_init  (void)
{
    uint8_t rl_count;
    rl_data.alm.trigger                     =       /* perform immediate alarm manager run (senseful if a running night blind service was interrupted by restart) */
    rl_data.alm.n_blind_req_bitlane         =
    rl_data.alm.n_blind_tmp_exclude_bitlane =
    rl_data.alm.n_blind_req_opc             =
    rl_data.alm.n_blind_req_delay           = 0;
    rl_data.alm.timer_page_index            = RL_RC_TIMER_PAGES;

    /* check the d_blind value for every blind */
    for(rl_count=0; rl_count<RL_COUNT; rl_count++)
    {
        rl_data.alm.shadowing_sm[rl_count].state = (nv_data.d_blind[rl_count].enable != 0) ? ALM_M_AUTO_MODE_INITIALIZED : ALM_M_AUTO_MODE_OFF;
    }
}

/* check the change of summer/winter time: this will be serviced at the April, 1st and Novwember, 1st at 1:00am */
/* on daylight saving time switch then adapt the shadowing time for every blind too */
static void  alm_manager_check_dst_switch (void)
{
    /* check daylight saving time switch at 1:00am */
    if(   (rl_data.rv3029.watch_el[RV3029_WATCH_EL_HOUR] == 1)
       && (rl_data.rv3029.watch_el[RV3029_WATCH_EL_MINUTE] == 0)
       && (rl_data.rv3029.watch_el[RV3029_WATCH_EL_DAY] == 1)
      )
    {
        if( (rl_data.rv3029.watch_el[RV3029_WATCH_EL_MONTH] == 4) && (rl_data.rv3029.dst_flag == 0) )
        {
            /* april 1st at 1:00am -> change into summertime */
            rv3029_set_hour(1);
        }
        else if( (rl_data.rv3029.watch_el[RV3029_WATCH_EL_MONTH] == 11) && (rl_data.rv3029.dst_flag == 1) )
        {
            /* november 1st at 1:00am -> change into wintertime */
            rv3029_set_hour(255);
        }
    }
}


/* check the night blind times and set a blind request to the motor if due */
/* consider deviating times for desired windows with an light bow during christmas time */
static void  alm_manager_check_n_blind_request (void)
{
  uint16_t consider_xmas;
  uint8_t alm_page_index = rl_data.alm.current_page;
  register uint8_t hour, minute;

  uint8_t * alm_page;

  minute = rl_data.rv3029.watch_el[RV3029_WATCH_EL_MINUTE];
  hour   = rl_data.rv3029.watch_el[RV3029_WATCH_EL_HOUR];

  /* if we have to consider xmas we AND lightbow windows with nv_data.n_blind_window.included               */
  /* so that a window will not be controlled by xmas bitlane if it is not included in night blind generally */
  consider_xmas = (nv_data.misc.properties & RL_PROP_SUPPORT_ADVENT) ? (nv_data.adv.has_lightbow & nv_data.n_blind_window.included) : 0;

  alm_page = (uint8_t *) & nv_data.n_blind[alm_page_index];

  if( (alm_page[MENU_N_BLIND_HOUR_UP] == hour) && (alm_page[MENU_N_BLIND_MINUTE_UP] == minute) )
  {
    rl_data.alm.n_blind_req_bitlane = nv_data.n_blind_window.included & ~consider_xmas;

    rl_data.alm.n_blind_req_opc = MOTOR_OPC_UP;
  }
  else if( (alm_page[MENU_N_BLIND_HOUR_DOWN] == hour) && (alm_page[MENU_N_BLIND_MINUTE_DOWN] == minute) )
  {
    rl_data.alm.n_blind_req_bitlane = nv_data.n_blind_window.included & ~consider_xmas;

    rl_data.alm.n_blind_req_opc = MOTOR_OPC_DOWN;
  }
  else if (nv_data.misc.properties & RL_PROP_SUPPORT_ADVENT)
  {
    NV_STRUC_N_BLIND tmp_page;
    tmp_page.hour_up     = nv_data.adv.morning_hour_on;
    tmp_page.minute_up   = nv_data.adv.morning_minute_on;
    tmp_page.hour_down   = nv_data.adv.evening_hour_off;
    tmp_page.minute_down = nv_data.adv.evening_minute_off;

    alm_page = (uint8_t *) & tmp_page;

    /* check the n_blind for Christmas */
    if( (alm_page[MENU_N_BLIND_HOUR_UP] == hour) && (alm_page[MENU_N_BLIND_MINUTE_UP] == minute) )
    {
      rl_data.alm.n_blind_req_bitlane = consider_xmas;

      rl_data.alm.n_blind_req_opc = MOTOR_OPC_UP;
    }
    else if( (alm_page[MENU_N_BLIND_HOUR_DOWN] == hour) && (alm_page[MENU_N_BLIND_MINUTE_DOWN] == minute) )
    {
      rl_data.alm.n_blind_req_bitlane = consider_xmas;

      rl_data.alm.n_blind_req_opc = MOTOR_OPC_DOWN;

    }
  }

  rl_data.alm.n_blind_req_delay = 0;
}

/* perform a motor request */
void  alm_manager_perform_n_blind_request_if_due (void)
{
  MOTOR_REQ req;

  if( (rl_data.alm.n_blind_req_bitlane != 0) && (rl_data.alm.n_blind_req_delay == 0) )
  {
    uint8_t rl_number;

    if /* the service mode is running */
       (rl_data.key.service_mode.entered != SERVICE_MODE_OFF)
    {
        /* do not include the window that is under service mode control */
        rl_data.alm.n_blind_req_bitlane &= ~_BV(rl_data.key.service_mode.button);
    }

    for(rl_number=RL_COUNT; rl_number<RL_COUNT_FULL; rl_number++)
    {
        if( rl_data.alm.n_blind_req_bitlane & (_BV(rl_number)) )
        {
            rl_data.alm.n_blind_req_bitlane = (rl_data.alm.n_blind_req_bitlane & ( ~_BV(rl_number) ));

            /* for extension rl we support night blind via remote control */
            rc_tx_send_command((RC_COMMAND *) &nv_data.motor[rl_number].rc_code[0], (rl_data.alm.n_blind_req_opc == MOTOR_OPC_UP) ? RC_DIGIT_VALUE_ON_FORCE : RC_DIGIT_VALUE_OFF_FORCE, RC_TX_DEFAULT_REPETITION_CNT);
        }
    }

    for(rl_number=0; rl_number<RL_COUNT; rl_number++)
    {
      if( rl_data.alm.n_blind_req_bitlane & (_BV(rl_number)) )
      {
        rl_data.alm.n_blind_req_bitlane = (rl_data.alm.n_blind_req_bitlane & ( ~_BV(rl_number) ));

        if( !(rl_data.alm.n_blind_tmp_exclude_bitlane & _BV(rl_number)) )
        {
            if(rl_data.alm.n_blind_req_opc == MOTOR_OPC_DOWN)
            {
              rl_data.alm.shadowing_sm[rl_number].state = ALM_M_AUTO_MODE_OFF;
            }

            req.opc       = rl_data.alm.n_blind_req_opc;
            req.rl_number = rl_number;
            req.runtime   = MOTOR_RUNTIME_FULL;
            motor_request(&req);

            rl_data.alm.n_blind_req_delay = 20;

            return;
        }
        else
        {
            rl_data.alm.n_blind_tmp_exclude_bitlane &= ~_BV(rl_number);
        }
      }
    }
  }
}


/* check a day blind for each window, consider individual time area, bright hysteresis and time hysteresis for every window */
static void  alm_manager_check_d_blind_request (void)
{
    MOTOR_REQ req;

    uint8_t rl_count;

    uint16_t now_time = ((uint16_t) rl_data.rv3029.watch_el[RV3029_WATCH_EL_HOUR] * 59) + rl_data.rv3029.watch_el[RV3029_WATCH_EL_MINUTE];

    /* check the d_blind value for every blind */
    for(rl_count=0; rl_count<RL_COUNT; rl_count++)
    {
        NV_STRUC_D_BLIND * current_d_blind = (NV_STRUC_D_BLIND *) &nv_data.d_blind[rl_count];

        if( (current_d_blind->enable != 0) && (rl_data.key.service_mode.button != rl_count) ) /* the blind is included for daytime shadowing */
        {
            ALM_SHADOWING_SM * shadowing_sm = (ALM_SHADOWING_SM *) &rl_data.alm.shadowing_sm[rl_count];

            uint16_t begin_time, end_time;

            begin_time = ((uint16_t) current_d_blind->hour_down * 59) + current_d_blind->minute_down;
            end_time   = ((uint16_t) current_d_blind->hour_up * 59) + current_d_blind->minute_up;

            if( now_time == begin_time )
            {
                rl_data.alm.shadowing_sm[rl_count].state = ((nv_data.d_blind[rl_count].enable != 0) ? ALM_M_AUTO_MODE_INITIALIZED : ALM_M_AUTO_MODE_OFF);
            }

            if( (begin_time <= now_time) && (now_time < end_time) )
            {
                uint8_t time_hysteresis;

                if( shadowing_sm->state == ALM_M_AUTO_MODE_INITIALIZED )
                {
                    shadowing_sm->state = ALM_M_AUTO_MODE_UPDRIVEN; /* initial state for day blind: blind is up */
                    shadowing_sm->sensor_upper_bound =
                    shadowing_sm->sensor_lower_bound = 0;
                }

                time_hysteresis        = current_d_blind->time_hyst;

                if(rl_data.sensor.single[rl_count] >= current_d_blind->sensor_down )
                {
                    if( (shadowing_sm->state == ALM_M_AUTO_MODE_UPDRIVEN) && (++shadowing_sm->sensor_upper_bound == time_hysteresis) )
                    {
                        shadowing_sm->state = ALM_M_AUTO_MODE_DRIVING_TO_HIDDEN_SENSOR;
                        shadowing_sm->sensor_lower_bound = 0;
                        req.opc = MOTOR_OPC_DOWN;

                        req.rl_number   = rl_count;
                        req.runtime     = MOTOR_RUNTIME_FULL;

                        motor_request(&req);
                    }
                }
                else if(rl_data.sensor.single[rl_count] < current_d_blind->sensor_up )
                {
                    if( (shadowing_sm->state == ALM_M_AUTO_MODE_DOWNDRIVEN) && (++shadowing_sm->sensor_lower_bound == time_hysteresis) )
                    {
                        shadowing_sm->state = ALM_M_AUTO_MODE_UPDRIVEN;
                        shadowing_sm->sensor_upper_bound = 0;
                        req.opc = MOTOR_OPC_UP;

                        req.rl_number   = rl_count;
                        req.runtime     = MOTOR_RUNTIME_FULL;

                        motor_request(&req);
                    }
                }
            }
            else if( now_time == end_time )
            {
                if(shadowing_sm->state == ALM_M_AUTO_MODE_DOWNDRIVEN)
                {
                    shadowing_sm->state = ALM_M_AUTO_MODE_UPDRIVEN;

                    req.opc = MOTOR_OPC_UP;

                    /* ignore sensor when day blind shall drive up */
                    req.rl_number   = rl_count;
                    req.runtime     = MOTOR_RUNTIME_FULL;

                    motor_request(&req);
                }
            }
        }
    }
}

void  alm_manager_perform_d_blind_request_callback (void)
{
  MOTOR_REQ req;

  uint8_t rl_count;

  /* check the d_blind value for every blind */
    for(rl_count=0; rl_count<RL_COUNT; rl_count++)
    {
        ALM_SHADOWING_SM * shadowing_sm = (ALM_SHADOWING_SM *) &rl_data.alm.shadowing_sm[rl_count];

        if( shadowing_sm->state == ALM_M_AUTO_MODE_DRIVING_TO_HIDDEN_SENSOR )
        {
            if(rl_data.sensor.single[rl_count] < SENSOR_HIDDEN_BY_BLIND_VALUE)
            {
                shadowing_sm->state = ALM_M_AUTO_MODE_DRIVING_TO_UNHIDDEN_SENSOR;

                req.opc = MOTOR_OPC_UP;

                /* ignore sensor when day blind shall drive up */
                req.rl_number   = rl_count;
                req.runtime     = MOTOR_RUNTIME_FULL;

                motor_request(&req);
            }
        }
        else if( shadowing_sm->state == ALM_M_AUTO_MODE_DRIVING_TO_UNHIDDEN_SENSOR )
        {
            if(rl_data.sensor.single[rl_count] >= nv_data.d_blind[rl_count].sensor_down)
            {
                shadowing_sm->state = ALM_M_AUTO_MODE_DOWNDRIVEN;

                req.opc = MOTOR_OPC_HALT;

                /* ignore sensor when day blind shall drive up */
                req.rl_number   = rl_count;
                req.runtime     = MOTOR_RUNTIME_FULL;

                motor_request(&req);
            }
        }
    }
}


static void  alm_manager_exec_timer_req_if_due (uint8_t * time_ptr, uint8_t * command_ptr)
{
    uint8_t command;

    register uint8_t hour, minute;

    /* read current time and weekday */
    minute = rl_data.rv3029.watch_el[RV3029_WATCH_EL_MINUTE];
    hour   = rl_data.rv3029.watch_el[RV3029_WATCH_EL_HOUR];

    if( (hour == * (time_ptr))  && (minute == * (time_ptr+1)) )
    {
         /* we found a timer request where to send a device ON telegram */
        command = RC_DIGIT_VALUE_ON;
    }
    else if( (hour == * (time_ptr+2))  && (minute == * (time_ptr+3)) )
    {
        /* we found a timer request where to send a device OFF telegram */
        command = RC_DIGIT_VALUE_OFF;
    }
    else
    {
        command = RC_DIGIT_VALUE_NO;
    }

    if(command != RC_DIGIT_VALUE_NO)
    {
        rc_tx_send_command((RC_COMMAND *) command_ptr, command, RC_TX_DEFAULT_REPETITION_CNT);
    }
}


/* rc timer request state machine */
static void  alm_manager_check_timer_request (void)
{
    uint8_t timer_page;
    if( nv_data.misc.properties & RL_PROP_SUPPORT_ADVENT )
    {
          alm_manager_exec_timer_req_if_due((uint8_t *) &nv_data.adv.morning_hour_on, (uint8_t *) &nv_data.adv.rc_code[0]);
          alm_manager_exec_timer_req_if_due((uint8_t *) &nv_data.adv.evening_hour_on, (uint8_t *) &nv_data.adv.rc_code[0]);
    }

    register uint8_t weekday = (rl_data.rv3029.watch_el[RV3029_WATCH_EL_WEEKDAY] & 0x07) - 1;

    for(timer_page=0; timer_page<RL_RC_TIMER_PAGES; timer_page++)
    {
          if( nv_data.timer[timer_page].weekdays & _BV(weekday) )
          {
              alm_manager_exec_timer_req_if_due( (uint8_t *) &nv_data.timer[timer_page].hour_on, (uint8_t *) &nv_data.timer[timer_page].rc_code[0]);
          }
    }
}


/* main call of alarm manager every minute */
void  alm_manager_check_request(void)
{
    if( rl_data.alm.trigger == 0 )
    {
      uint8_t tmp_page = rl_data.alm.current_page;

      /* get current time to compare it with the setting times */
      rv3029_read_time();

      rl_data.alm.trigger = 61-rl_data.time.current_second;

      n_blind_day_month_to_page__(rl_data.rv3029.watch_el[RV3029_WATCH_EL_DAY], rl_data.rv3029.watch_el[RV3029_WATCH_EL_MONTH], rl_data.alm.current_page);

      if(tmp_page != rl_data.alm.current_page)
      {
          /* update current n_blind page (it happens after every half month) */
          rl_data.menu.n_blind.current_page = rl_data.alm.current_page;
      }

      /* after 1 minute we check also timer requests of remote control */
      rl_data.alm.timer_page_index = 0;

      /* ckeck for post-/preset of the watch due to daylight saving time switch */
      alm_manager_check_dst_switch();

      /* ckeck for possible time congruence of night blind times and set a motor_request if due */
      if(nv_data.misc.properties & RL_PROP_SUPPORT_N_BLIND)
      {
          alm_manager_check_n_blind_request();
      }

      /* ckeck for possible time congruence of day blind times and set a motor_request if due   */
      alm_manager_check_d_blind_request();

      alm_manager_check_timer_request();
    }

    alm_manager_perform_n_blind_request_if_due();
}

/************************************************************************/
/* copyright: BBC-Hoerer 2016-2018                                      */
/************************************************************************/

