/******************************************************************************/
/* Project:   blind control set                                               */
/* Author:    BBC-Hoerer                                                      */
/*            You may use or modify this code for your purpose.               */
/*            BBC-Hoerer is not responsible for consequences using this code. */
/******************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "i2cmaster.h"
#include "lcd.h"
#include "_rl_com.h"

/* servicing keys, those are: the keypad of the hmi and the up/down wallbuttons */
#define   KEY_STATE_DEBOUNCE_DONE    0
#define   KEY_STATE_DEBOUNCE_VALUE   4     // 40ms
#define   KEY_STATE_DEBOUNCE_OFF     0xFF

#define   KEY_STATE_RELEASED     0
#define   KEY_STATE_PRESSED      1
#define   KEY_STATE_LONGPRESSED  150       // 1500ms
#define   KEY_STATE_TOGGLE_UPPER 165       // additional 150ms

#define   KEY_LONGPRESS        100         // 1sec

#if defined CPU_ATMEGA16

  #define   KEY_WB_LOW         !( PINC & (1<<PINC3) )
  #define   KEY_KP_LOW         !( PINC & (1<<PINC2) )

#elif defined CPU_ATMEGA168

  #define   KEY_WB_LOW        !( PIND & (1<<PIND3) )
  #define   KEY_KP_LOW        !( PIND & (1<<PIND4) )

#endif

/* init function of key service */
void key_init	(void)
{
  uint8_t bank, i;

  for(bank=0;bank<KEY_BANK_MAX;bank++)
  {
    for(i=0;i<8;i++)
    {
      rl_data.key.bank[bank].state[i] = KEY_STATE_RELEASED;
    }
    rl_data.key.bank[bank].input    = 0;
  }

  rl_data.key.wb_debounce_state     =
  rl_data.key.kp_debounce_state     = KEY_STATE_DEBOUNCE_OFF;
  rl_data.key.wb_trigger_read_input =
  rl_data.key.kp_trigger_read_input =
  rl_data.key.evaluate              = 0;


  //	set addressing style
	mcp23017_write_register(MCP23017_ADDRESS_WB+MCP23017_WB_UP_BANK+MCP23017_IOCON, 0xC0);   /* INTA and INTB are internally connected */

  _delay_ms(5);                  // delay for rearranging register addresses

  mcp23017_init_for_input(MCP23017_ADDRESS_WB+MCP23017_WB_UP_BANK);
  mcp23017_init_for_input(MCP23017_ADDRESS_WB+MCP23017_WB_DOWN_BANK);
}

/* this function is called within an 10ms interrupt: it debounces the key input from the mcp23017 and triggers independendly of that a key evaluation in the main routine */
void key_timer_10ms (void)
{
    /* check if we need to read no keys from mcp23017 */
    if( (rl_data.key.wb_debounce_state != KEY_STATE_DEBOUNCE_DONE) && (rl_data.key.wb_debounce_state != KEY_STATE_DEBOUNCE_OFF) )
    {
        if( --rl_data.key.wb_debounce_state == KEY_STATE_DEBOUNCE_DONE )
        {
          rl_data.key.wb_trigger_read_input = 1;
        }
    }

    if( (rl_data.key.kp_debounce_state != KEY_STATE_DEBOUNCE_DONE) && (rl_data.key.kp_debounce_state != KEY_STATE_DEBOUNCE_OFF) )
    {
        if( --rl_data.key.kp_debounce_state == KEY_STATE_DEBOUNCE_DONE )
        {
          rl_data.key.kp_trigger_read_input = 1;
        }
    }

    /* we evaluate the keys every 10ms */
    rl_data.key.evaluate = 1;
}


/* this function is called every 10ms in the main loop. It reads the associated input of a mcp23017 after a keys are debounced */
/* Morover this function deliveres for every key bench if a key was pressed (also repeatedly), longpressed or released.        */
/* Key benches are the keypad of hmi the up keys and the down key of wallbutton)                                               */
void key_evaluate (void)
{
    uint8_t bank;

    if( KEY_WB_LOW && (rl_data.key.wb_debounce_state == KEY_STATE_DEBOUNCE_OFF) )
    {
        /* we check the h/l-flanc  of wall buttons (up/down) mcp23017 */
        rl_data.key.wb_debounce_state = KEY_STATE_DEBOUNCE_VALUE;
    }

    if( KEY_KP_LOW && (rl_data.key.kp_debounce_state == KEY_STATE_DEBOUNCE_OFF) )
    {
      /* we check the h/l-flanc  of keypad mcp23017 */
      rl_data.key.kp_debounce_state = KEY_STATE_DEBOUNCE_VALUE;
    }

    if( rl_data.key.wb_trigger_read_input == 1 )
    {
      rl_data.key.wb_trigger_read_input = 0;

      rl_data.key.wb_debounce_state = KEY_STATE_DEBOUNCE_OFF;

      /* we read the debounced keys from mcp23017 input */
      rl_data.key.bank[WB_UP_INDEX].input   = mcp23017_in(MCP23017_ADDRESS_WB+MCP23017_WB_UP_BANK);
      rl_data.key.bank[WB_DOWN_INDEX].input = mcp23017_in(MCP23017_ADDRESS_WB+MCP23017_WB_DOWN_BANK);
    }

    if( rl_data.key.kp_trigger_read_input == 1 )
    {
      rl_data.key.kp_trigger_read_input = 0;

      rl_data.key.kp_debounce_state = KEY_STATE_DEBOUNCE_OFF;

      /* we read the debounced keys from mcp23017 input */
      rl_data.key.bank[KP_INDEX].input = mcp23017_in(MCP23017_ADDRESS_HMI+MCP23017_KP_BANK) & 0x7F;
    }

    for(bank=0;bank<KEY_BANK_MAX;bank++)
    {
      rl_data.key.bank[bank].sum = rl_data.key.bank[bank].released = rl_data.key.bank[bank].pressed = rl_data.key.bank[bank].longpressed = 0;
    }

    if( rl_data.key.evaluate == 1 )
    {
        uint8_t i, keybit;

        rl_data.key.evaluate = 0;

        service_mode_entering_intervall_reset();

        for(bank=0;bank<KEY_BANK_MAX;bank++)
        {
            KEY_BANK * key_bank = (KEY_BANK *) & rl_data.key.bank[bank];
            uint8_t input;

            keybit = 0x01;

            /* include remote control key input */
            input = (key_bank->input | rl_data.rc_rx.key[bank]);

            for(i=0;i<8;i++)
            {
                if( (key_bank->state[i] != KEY_STATE_RELEASED) && !(input & keybit) )
                {
                  key_bank->state[i] = KEY_STATE_RELEASED;

                  key_bank->sum      |= keybit;
                  key_bank->released |= keybit;

                  service_mode_check_enter_exit(bank, i);

                }
                else if( input & keybit )
                {
                    key_bank->state[i]++;

                    if( key_bank->state[i] == KEY_STATE_PRESSED )
                    {
                      key_bank->sum |= keybit;
                      key_bank->pressed |= keybit;
                    }
                    else if( key_bank->state[i] == KEY_STATE_TOGGLE_UPPER )
                    {
                        key_bank->sum |= keybit;
                        key_bank->pressed |= keybit;    // repeated
                        key_bank->state[i] = KEY_STATE_LONGPRESSED;
                    }
                    else
                    {
                        if(key_bank->state[i]>= KEY_STATE_LONGPRESSED)
                        {
                          key_bank->sum |= keybit;
                          key_bank->longpressed |= keybit;
                        }
                    }
                }

                keybit = keybit<<1;
            }
        }
    }
}

/************************************************************************/
/* copyright: BBC-Hoerer 2016-2018                                      */
/************************************************************************/
