/****************************************************************************
 Title:     HD44780U LCD library
 Author:    Peter Fleury <pfleury@gmx.ch>  http://tinyurl.com/peterfleury
 File:	    $Id: lcd.c,v 1.15.2.2 2015/01/17 12:16:05 peter Exp $
 Software:  AVR-GCC 3.3
 Target:    any AVR device, memory mapped mode only for AT90S4414/8515/Mega

 DESCRIPTION
       Basic routines for interfacing a HD44780U-based text lcd display

       Originally based on Volker Oth's lcd library,
       changed lcd_init(), added additional constants for lcd_command(),
       added 4-bit I/O mode, improved and optimized code.

       Library can be operated in memory mapped mode (LCD_IO_MODE=0) or in
       4-bit IO port mode (LCD_IO_MODE=1). 8-bit IO port mode not supported.

       Memory mapped mode compatible with Kanda STK200, but supports also
       generation of R/W signal through A8 address line.

 USAGE
       See the C include lcd.h file for a description of each function

*****************************************************************************/
#include <inttypes.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <stdlib.h>
#include "lcd.h"
#include "_rl_com.h"

//#define LCD_NO_INPUT

#define LCD_CG_CHAR   4

const PROGMEM unsigned char cg_add[] =   //  up to eight self defined symbols
{
  0x00, 0x08, 0x0c, 0x0e, 0x0c, 0x08, 0x00, 0x00,         // LCD_CHAR_FAT_ARROW_RIGHT
  0x04, 0x0a, 0x11, 0x1f, 0x11, 0x0a, 0x04, 0x00,         // LCD_CHAR_FAT_ARROW_UP_DOWN
  0x00, 0x01, 0x03, 0x16, 0x1c, 0x08, 0x00, 0x00,         // LCD_CHAR_HAKEN
  //0x04, 0x0e, 0x0a, 0x0a, 0x11, 0x1f, 0x04, 0x00,         // LCD_CHAR_JINGLEBELL

  //0x4,0xa,0x1f,0x4,0x4,0x1f,0xa,0x4,
  //0x04, 0x0e, 0x1f, 0x04, 0x04, 0x1f, 0x0e, 0x04,         // fat arrow up/down
  //0x04, 0x0e, 0x15, 0x04, 0x04, 0x15, 0x0e, 0x04,         // arrow up/down
};

static uint8_t           lcd_port;

#define DDR(x) (*(&x - 1))      /* address of data direction register of port x */
#define PIN(x) (*(&x - 2))    /* address of input register of port x          */


#define lcd_e_delay()   _delay_us(LCD_DELAY_ENABLE_PULSE)
#define lcd_e_high()    LCD_PORT  |=  _BV(LCD_E_PIN);
#define lcd_e_low()     LCD_PORT  &= ~_BV(LCD_E_PIN);
#define lcd_e_toggle()  toggle_e()
#define lcd_rw_high()   LCD_PORT |=  _BV(LCD_RW_PIN)
#define lcd_rw_low()    LCD_PORT &= ~_BV(LCD_RW_PIN)
#define lcd_rs_high()   LCD_PORT |=  _BV(LCD_RS_PIN)
#define lcd_rs_low()    LCD_PORT &= ~_BV(LCD_RS_PIN)

#if LCD_LINES==1
#define LCD_FUNCTION_DEFAULT    LCD_FUNCTION_4BIT_1LINE
#else
#define LCD_FUNCTION_DEFAULT    LCD_FUNCTION_4BIT_2LINES
#endif

#if LCD_CONTROLLER_KS0073==1
#if LCD_LINES==4

#define KS0073_EXTENDED_FUNCTION_REGISTER_ON  0x2C   /* |0|010|1100 4-bit mode, extension-bit RE = 1 */
#define KS0073_EXTENDED_FUNCTION_REGISTER_OFF 0x28   /* |0|010|1000 4-bit mode, extension-bit RE = 0 */
#define KS0073_4LINES_MODE                    0x09   /* |0|000|1001 4 lines mode */

#endif
#endif

/*
** function prototypes
*/
static void toggle_e(void);

void lcd_backlight_off (void)
{
  LCD_PORT |=  _BV(LCD_BL_PIN);
  mcp23017_out(MCP23017_ADDRESS_HMI+MCP23017_LCD_BANK, LCD_PORT);
}

void lcd_backlight_on  (void)
{
  LCD_PORT &= ~_BV(LCD_BL_PIN);
  mcp23017_out(MCP23017_ADDRESS_HMI+MCP23017_LCD_BANK, LCD_PORT);
}


/*************************************************************************
delay for a minimum of <us> microseconds
the number of loops is calculated at compile-time from MCU clock frequency
*************************************************************************/
#define delay(us)  _delay_us(us)


/* toggle Enable Pin to initiate write */
static void toggle_e(void)
{
    lcd_e_high();
    mcp23017_out(MCP23017_ADDRESS_HMI+MCP23017_LCD_BANK, LCD_PORT);
    lcd_e_delay();
    lcd_e_low();
    mcp23017_out(MCP23017_ADDRESS_HMI+MCP23017_LCD_BANK, LCD_PORT);
}


/*************************************************************************
Low-level function to write byte to LCD controller
Input:    data   byte to write to LCD
          rs     1: write data
                 0: write instruction
Returns:  none
*************************************************************************/
static void lcd_write(uint8_t data,uint8_t rs)
{
  unsigned char dataBits ;

  if (rs)
  {        /* write data        (RS=1, RW=0) */
    lcd_rs_high();
  }
  else
  {         /* write instruction (RS=0, RW=0) */
    lcd_rs_low();
  }

  lcd_rw_low();    /* RW=0  write mode      */

#if !defined LCD_NO_INPUT
  /* set all to output */
  mcp23017_write_register(MCP23017_ADDRESS_HMI+MCP23017_LCD_BANK+MCP23017_IODIR, 0x00);

#endif

  /* output high nibble first */
  dataBits = LCD_PORT & 0xF0;
  LCD_PORT = dataBits |((data>>4)&0x0F);
  lcd_e_toggle();

  /* output low nibble */
  LCD_PORT = dataBits | (data&0x0F);
  lcd_e_toggle();

  /* all data pins high (inactive) */
  LCD_PORT = dataBits | 0x0F;
}


#if !defined LCD_NO_INPUT
/*************************************************************************
Low-level function to read byte from LCD controller
Input:    rs     1: read data
                 0: read busy flag / address counter
Returns:  byte read from LCD controller
*************************************************************************/
static uint8_t lcd_read(uint8_t rs)
{
    uint8_t data;

    if (rs)
        lcd_rs_high();                       /* RS=1: read data      */
    else
        lcd_rs_low();                        /* RS=0: read busy flag */
    lcd_rw_high();                           /* RW=1  read mode      */

    mcp23017_out(MCP23017_ADDRESS_HMI+MCP23017_LCD_BANK, LCD_PORT);

    /* set data0..datas3 to input, rest of the lines remains output */
    mcp23017_write_register(MCP23017_ADDRESS_HMI+MCP23017_LCD_BANK+MCP23017_IODIR, 0x0F);

    lcd_e_high();
    mcp23017_out(MCP23017_ADDRESS_HMI+MCP23017_LCD_BANK, LCD_PORT);
    lcd_e_delay();
    data = (mcp23017_in(MCP23017_ADDRESS_HMI+MCP23017_LCD_BANK) << 4);     /* read high nibble first */
    lcd_e_low();
    mcp23017_out(MCP23017_ADDRESS_HMI+MCP23017_LCD_BANK, LCD_PORT);
    lcd_e_delay();                       /* Enable 500ns low       */

    lcd_e_high();
    mcp23017_out(MCP23017_ADDRESS_HMI+MCP23017_LCD_BANK, LCD_PORT);
    lcd_e_delay();
    data |= (mcp23017_in(MCP23017_ADDRESS_HMI+MCP23017_LCD_BANK) & 0x0F);    /* read low nibble        */
    lcd_e_low();
    mcp23017_out(MCP23017_ADDRESS_HMI+MCP23017_LCD_BANK, LCD_PORT);
    return data;
}
#endif

/*************************************************************************
loops while lcd is busy, returns address counter
*************************************************************************/
static uint8_t lcd_waitbusy(void)
{
#if defined LCD_NO_INPUT

    delay(700);

    return(0);

#else

    register uint8_t c;

    /* wait until busy flag is cleared */
    while ( (c=lcd_read(0)) & (1<<LCD_BUSY)) {}

    /* the address counter is updated 4us after the busy flag is cleared */
    delay(LCD_DELAY_BUSY_FLAG);

    /* now read the address counter */
    return (lcd_read(0));  // return address counter
#endif
}


/*
** PUBLIC FUNCTIONS
*/

/*************************************************************************
Send LCD controller instruction command
Input:   instruction to send to LCD controller, see HD44780 data sheet
Returns: none
*************************************************************************/
void lcd_command(uint8_t cmd)
{
    lcd_waitbusy();
    lcd_write(cmd,0);
}


/*************************************************************************
Send data byte to LCD controller
Input:   data to send to LCD controller, see HD44780 data sheet
Returns: none
*************************************************************************/
void lcd_data(uint8_t data)
{
    lcd_waitbusy();
    lcd_write(data,1);
}



/*************************************************************************
Set cursor to specified position
Input:    x  horizontal position  (0: left most position)
          y  vertical position    (0: first line)
Returns:  none
*************************************************************************/
void lcd_gotoxy(uint8_t x, uint8_t y)
{
#if LCD_LINES==1
    lcd_command((1<<LCD_DDRAM)+LCD_START_LINE1+x);
#endif
#if LCD_LINES==2
    if ( y==0 )
        lcd_command((1<<LCD_DDRAM)+LCD_START_LINE1+x);
    else
        lcd_command((1<<LCD_DDRAM)+LCD_START_LINE2+x);
#endif
#if LCD_LINES==4
    if ( y==0 )
	{
        lcd_command((1<<LCD_DDRAM)+LCD_START_LINE1+x);
	}
    else if ( y==1 )
	{
        lcd_command((1<<LCD_DDRAM)+LCD_START_LINE2+x);
	}
    else if ( y==2 )
	{
        lcd_command((1<<LCD_DDRAM)+LCD_START_LINE3+x);
	}
    else /* y==3 */
	{
        lcd_command((1<<LCD_DDRAM)+LCD_START_LINE4+x);
	}
#endif

}/* lcd_gotoxy */


/*************************************************************************
Clear display and set cursor to home position
*************************************************************************/
void lcd_clrscr(void)
{
    lcd_command(1<<LCD_CLR);
#if defined LCD_NO_INPUT
    delay(700);
#endif
}

#if 0
/*************************************************************************
Set cursor to home position
*************************************************************************/
void lcd_home(void)
{
    lcd_command(1<<LCD_HOME);
}
#endif

/*************************************************************************
write a character from data memory to actual display position
*************************************************************************/
static void lcd_putc_local(char c)
{
    lcd_waitbusy();   // read busy-flag and address counter

    #if LCD_WRAP_LINES==1
    #if LCD_LINES==1
            if ( pos == LCD_START_LINE1+LCD_DISP_LENGTH ) {
                lcd_write((1<<LCD_DDRAM)+LCD_START_LINE1,0);
            }
    #elif LCD_LINES==2
            if ( pos == LCD_START_LINE1+LCD_DISP_LENGTH ) {
                lcd_write((1<<LCD_DDRAM)+LCD_START_LINE2,0);
            }else if ( pos == LCD_START_LINE2+LCD_DISP_LENGTH ){
                lcd_write((1<<LCD_DDRAM)+LCD_START_LINE1,0);
            }
    #elif LCD_LINES==4
            if ( pos == LCD_START_LINE1+LCD_DISP_LENGTH )
        {
                lcd_write((1<<LCD_DDRAM)+LCD_START_LINE2,0);
            }
        else if ( pos == LCD_START_LINE2+LCD_DISP_LENGTH )
        {
                lcd_write((1<<LCD_DDRAM)+LCD_START_LINE3,0);
            }
        else if ( pos == LCD_START_LINE3+LCD_DISP_LENGTH )
        {
                lcd_write((1<<LCD_DDRAM)+LCD_START_LINE4,0);
            }
        else if ( pos == LCD_START_LINE4+LCD_DISP_LENGTH )
        {
                lcd_write((1<<LCD_DDRAM)+LCD_START_LINE1,0);
            }
    #endif
            lcd_waitbusy();
    #endif

    lcd_write(c, 1);
}


/*************************************************************************
write a character from data memory to given display position
*************************************************************************/
void lcd_putc(uint8_t x, uint8_t y, char c)
{
  lcd_gotoxy(x, y);
  lcd_putc_local(c);
}
/*************************************************************************
write a string from data memory to display position
*************************************************************************/
void lcd_puts(uint8_t x, uint8_t y, const char *s)
{
	  /* print string on lcd (no auto linefeed) */
    register char c;

    lcd_gotoxy( x, y);

    /* read  string from ram */
    while ( (c = *s++) )
    {
      lcd_putc_local(c);
    }
}


/*************************************************************************
write a string from PROGMEM memory to display position
*************************************************************************/
void lcd_puts_p(uint8_t x, uint8_t y, const char *s)
{
	/* print string on lcd (no auto linefeed) */
    register char c;

    lcd_gotoxy(x, y);

    /* read constant string from program memory */
    while ( (c = pgm_read_byte(s++)) )
    {
      lcd_putc_local(c);
    }
}


/*************************************************************************
write a 2-digit decimal (<100) value with leading zero to display position
*************************************************************************/
void lcd_put_two_digit_dec_u8(uint8_t x, uint8_t y, uint8_t value)
{
  lcd_gotoxy(x, y);
  lcd_putc_local((value / 10) | 0x30);
  lcd_putc_local((value % 10) | 0x30);
}

/*************************************************************************
write a 3-digit decimal (<255) value with leading zero to display position
*************************************************************************/
void lcd_put_three_digit_dec_u8(uint8_t x, uint8_t y, uint8_t value)
{
  uint8_t tmp_value = value;
  uint8_t i = 2;
  do
  {
    lcd_putc(x+i, y, '0' + tmp_value%10); tmp_value/=10;
  } while (i--);
}

/*************************************************************************
write a 2-digit hexadecimal (<256) value with leading zero to display position
*************************************************************************/
void lcd_put_two_digit_hex_u8(uint8_t x, uint8_t y, uint8_t value)
{
  uint8_t tmp_value = value;
  uint8_t tmp_1;

  uint8_t i = 1;
  do
  {
    tmp_1 = tmp_value%16;
    tmp_1 = (tmp_1 > 9) ? (tmp_1-10) + 'A' : tmp_1 + '0';

    lcd_putc(x+i, y, tmp_1); tmp_value/=16;
  } while (i--);
}



/*************************************************************************
write a 4-digit decimal (<1024) value with leading zero to display position
*************************************************************************/
void lcd_put_four_digit_dec_u16(uint8_t x, uint8_t y, uint16_t value)
{
  uint16_t tmp_value = value;
  uint8_t i = 3;
  do
  {
    lcd_putc(x+i, y, '0' + tmp_value%10); tmp_value/=10;
  } while (i--);
}



/*************************************************************************
Initialize display and select type of cursor
Input:    dispAttr LCD_DISP_OFF            display off
                   LCD_DISP_ON             display on, cursor off
                   LCD_DISP_ON_CURSOR      display on, cursor on
                   LCD_DISP_CURSOR_BLINK   display on, cursor on flashing
Returns:  none
*************************************************************************/
void lcd_init(uint8_t dispAttr)
{
    /*
     *  Initialize LCD to 4 bit I/O mode
     */

  LCD_PORT = 0;

  /* set output direction */
  mcp23017_write_register(MCP23017_ADDRESS_HMI+MCP23017_LCD_BANK+MCP23017_IODIR, LCD_PORT);

  lcd_backlight_off();

  delay(LCD_DELAY_BOOTUP);             /* wait 16ms or more after power-on       */

  /* initial write to lcd is 8bit */
  LCD_PORT |= _BV(LCD_DATA1_PIN);    // LCD_FUNCTION>>4;
  LCD_PORT |= _BV(LCD_DATA0_PIN);    // LCD_FUNCTION_8BIT>>4;
  lcd_e_toggle();
  delay(LCD_DELAY_INIT);               /* delay, busy flag can't be checked here */

  /* repeat last command */
  lcd_e_toggle();
  delay(LCD_DELAY_INIT_REP);           /* delay, busy flag can't be checked here */

  /* repeat last command a third time */
  lcd_e_toggle();
  delay(LCD_DELAY_INIT_REP);           /* delay, busy flag can't be checked here */

  /* now configure for 4bit mode */
  LCD_PORT &= ~_BV(LCD_DATA0_PIN);   // LCD_FUNCTION_4BIT_1LINE>>4
  lcd_e_toggle();
  delay(LCD_DELAY_INIT_4BIT);          /* some displays need this additional delay */

  /* from now the LCD only accepts 4 bit I/O, we can use lcd_command() */

#if KS0073_4LINES_MODE
    /* Display with KS0073 controller requires special commands for enabling 4 line mode */
	lcd_command(KS0073_EXTENDED_FUNCTION_REGISTER_ON);
	lcd_command(KS0073_4LINES_MODE);
	lcd_command(KS0073_EXTENDED_FUNCTION_REGISTER_OFF);
#else
  lcd_command(LCD_FUNCTION_DEFAULT);      /* function set: display lines  */
#endif
  lcd_command(LCD_DISP_OFF);              /* display off                  */
  lcd_clrscr();                           /* display clear                */
  lcd_command(LCD_MODE_DEFAULT);          /* set entry mode               */
  lcd_command(dispAttr);                  /* display/cursor control       */

  /* load user defined characters into cg ram */
  {
    int i= 0; //Byte counter
    lcd_command(0x40); //CG-RAM Address = 0
    for(i= 0; i <LCD_CG_CHAR*8 ; i++) //Write up to 64 Bytes of up to 8 characters
    {
       lcd_data(pgm_read_byte(&cg_add[i]));
    }
    lcd_command(0x80); // Set DD-RAM Address = 0
  }
}


