/******************************************************************************/
/* Project:   blind control set                                               */
/* Author:    BBC-Hoerer                                                      */
/*            You may use or modify this code for your purpose.               */
/*            BBC-Hoerer is not responsible for consequences using this code. */
/******************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "i2cmaster.h"
#include "lcd.h"
#include "_rl_com.h"
#include <avr/wdt.h>

/* main routine and timer isr */
ISR(TIMER1_COMPA_vect)
{
  static uint8_t time_x100ms; /* divider 100ms */
  static uint8_t time_x1s;    /* divider 1s    */

  /* we are here every 10ms */

  /* this timer expires every 10ms and cares for key debouncing */
  key_timer_10ms();

  if ( time_x100ms )
  {
    time_x100ms--;
  }
  else
  {
    /* we are here every 100ms */
    rc_rx_timeout();

    time_x100ms = 9;

    if( rl_data.alm.n_blind_req_delay != 0 )
    {
      rl_data.alm.n_blind_req_delay--;
    }

    if ( time_x1s )
    {
      time_x1s--;
    }
    else
    {
      /* we are here every 1 second */

      motor_update_driving_time();

      rl_data.rc_tx.test_trigger = 1;

      time_x1s = 9;

      if(rl_data.time.current_second<59)
      {
        rl_data.time.current_second++;
      }

      if(rl_data.alm.trigger != 0)
      {
        rl_data.alm.trigger--;
      }
      rl_data.menu.curr_value.trigger = 1;
    }
  }
}


int main(void)
{
    volatile static uint16_t startup_ctr = 60000;
    uint8_t reset_flag;
    #if defined CPU_ATMEGA16
    reset_flag = MCUCSR & 0x1F;
        MCUCSR = 0x80;          /* ATMEGA16: jtag pins are used as regular i/o pin: switch off jtag by writing this command twice within four cycles, otherwise we read at least once a faulty input (KEY_KP_LOW) */
        MCUCSR = 0x80;
    #else
        reset_flag = 0;
    #endif

    #if defined CPU_ATMEGA16

      PORTC = 0x1c;          /* Pin C2, C3, C4 have pullup (Pin C2, C3 is MCP23017 interrupt input, Pin C4 is HMI detection input) */
      DDRC  = 0b01100000;    /* all Port C is set to input direction beside of PORTC5 */

    #elif defined CPU_ATMEGA168

      PORTD = 0x38;          /* Pin D3, D4, D5 have pullup (Pin D3, D4 is MCP23017 interrupt input, Pin D5 is HMI detection input) */
      DDRD  = 0b01000000;    /* all Port D is set to input direction  beside of PORTD6 */

    #endif

    while (startup_ctr--);

    eeprom_read_var_from_eeprom((uint8_t *) & nv_data, 0, sizeof(NV_DATA));

    i2c_init();

    motor_init();

    key_init();

    rv3029_init(reset_flag);

    service_mode_init();

    alm_manager_init();

    menu_init();

    wb_init();

    hmi_init();

    sensor_init();

#if defined CPU_ATMEGA16

    TCNT1  = 0;
    OCR1A  = 2500;        /* timer compare interrupt after each 0,01s */
    TCCR1A = 0b00000000;
    TCCR1B = 0b00001011;  /* prescaler 64; CPU-Clock */
    TIMSK  = 0b00010000;  /* interrupt at compare match */

#elif defined CPU_ATMEGA168

    TCNT1  = 0;
    OCR1A  = 2500;        /* timer compare interrupt after each 0,01s */
    TCCR1A = 0b00000000;
    TCCR1B = 0b00001011;  /* prescaler 64; CPU-Clock */
    TCCR1C = 0b00000000;
    TIMSK1 = 0b00000010;  /* interrupt at compare match */

#endif

    rc_init();

    REBOOT_LED_ON;

    wdt_reset();
    wdt_enable(WDTO_2S);

    sei();                  /* enable interrupts */

    for (;;)
    {
        rl_noinit_data.line = __LINE__;

        wdt_reset();

        rl_noinit_data.line = __LINE__;

        rc_rx_evaluate();                   /* evaluate key change of remote control */

        rl_noinit_data.line = __LINE__;

        hmi_pull_plug_check();

        rl_noinit_data.line = __LINE__;

        key_evaluate();                     /* evaluate key change */

        rl_noinit_data.line = __LINE__;

        alm_manager_check_request();        /* check for alarm requests (blind- or timer-requests) */

        rl_noinit_data.line = __LINE__;

        menu_srv();                         /* display the menu on request */

        rl_noinit_data.line = __LINE__;

        sensor_check_adc_conversion();      /* check if a conversion is completed */

        rl_noinit_data.line = __LINE__;

        wb_evaluate();                      /* check if a wall button (up/down) was pressed */

        rl_noinit_data.line = __LINE__;

        motor_check_new_request();          /* motor action (runtime check, d_blind check) */

        rl_noinit_data.line = __LINE__;

        rc_tx_command_scheduler();          /* check if a rc command is to be transmitted */
    }
}

/************************************************************************/
/* copyright: BBC-Hoerer 2016-2018                                      */
/************************************************************************/



