/******************************************************************************/
/* Project:   blind control set                                               */
/* Author:    BBC-Hoerer                                                      */
/*            You may use or modify this code for your purpose.               */
/*            BBC-Hoerer is not responsible for consequences using this code. */
/******************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "i2cmaster.h"
#include "lcd.h"
#include "_rl_com.h"

#define ROOM_x_NAME_SIZE   14

const  PROGMEM char menu_room_name[RL_COUNT_FULL][ROOM_x_NAME_SIZE] = {ROOM_1_NAME, ROOM_2_NAME, ROOM_3_NAME, ROOM_4_NAME, ROOM_5_NAME, ROOM_6_NAME, ROOM_7_NAME, ROOM_8_NAME, ROOM_9_NAME, ROOM_10_NAME, ROOM_11_NAME};

const  PROGMEM char menu_item_weekday[8][3] = {"", "So", "Mo", "Di", "Mi", "Do", "Fr", "Sa"};
const  PROGMEM char menu_item_on_off_cmd[5][7] = {"off   ", "off_F ", "on    ", "on_F  ", "on_off"};

typedef void (*VoidFnct) (void);

/* consider that the sequence of the functions must follow the enum "MENU_INDIZES" in _rl_com.h */
const PROGMEM VoidFnct menu_init_fct[] = {menu_n_blind_init, menu_d_blind_init, menu_motor_init, menu_adv_init, menu_rc_timer_init, menu_dt_init, menu_reboots_init, menu_misc_init, menu_curr_value_init, menu_main_init};
const PROGMEM VoidFnct menu_srv_fct[]  = {menu_n_blind_srv, menu_d_blind_srv, menu_motor_srv, menu_adv_srv, menu_rc_timer_srv, menu_dt_srv, menu_reboots_srv, menu_misc_srv, menu_curr_value_srv, menu_main_srv};


/* here are described the 1-byte variable types with their lower and upper bound (MENU_VAR_SENSOR is 16 bit and included for smoothly programming) */
const
struct
{
  uint8_t lower;
  uint8_t upper;
} menu_var_boundaries[MENU_VAR_BOUNDARY_CNT]  PROGMEM = {  {1, 7},                        /* MENU_VAR_WEEKDAY */
                                                           {1, 31},                       /* MENU_VAR_DAY     */
                                                           {1, 12},                       /* MENU_VAR_MONTH   */
                                                           {18, 79},                      /* MENU_VAR_YEAR    */
                                                           {0, 23},                       /* MENU_VAR_HOUR    */
                                                           {0, 59},                       /* MENU_VAR_MINUTE  */
                                                           {0, 59},                       /* MENU_VAR_SECOND  */
                                                           {0, 1},                        /* MENU_VAR_BOOLEAN    ja/nein selection */
                                                           {0, 1},                        /* MENU_VAR_BOOLEAN_02 ' ' / check selection */
                                                           {0, RL_COUNT-1},               /* MENU_VAR_RL_CNT  */
                                                           {0, RL_COUNT_FULL-1},          /* MENU_VAR_RL_CNT_FULL  */
                                                           {0, 255},                      /* MENU_VAR_SENSOR  */
                                                           {0, RL_N_BLIND_TIME_PAGES-1},  /* MENU_VAR_N_BLIND_PAGE (0..23 for 12 month) */
                                                           {0, RL_RC_TIMER_PAGES-1},      /* MENU_VAR_TIMER_PAGE */
                                                           {0, 9},                        /* MENU_VAR_SHADOWING_SM */
                                                           {0, 250},                      /* MENU_VAR_TEMPERATURE */
                                                           {0, 2},                        /* MENU_VAR_RC_SYS */
                                                           {0, 4},                        /* MENU_VAR_RC_DEV */
                                                           {0, 15},                       /* MENU_VAR_RC_CMD */
                                                        };

/* call the init function of all the menu pages */
void menu_init	(void)
{
  rl_data.menu.index = MENU_INDEX_MAX;

  /* main menu initialisation */
  menu_main_coldstart_init();

  /* night blind menu initialisation */
  menu_n_blind_coldstart_init();

  /* day blind menu initialisation */
  menu_d_blind_coldstart_init();

  /* motor menu initialisation */
  menu_motor_coldstart_init();

#if defined RC_SUPPORT_TX
  /* remote control timer menu initialisation */
  menu_rc_timer_coldstart_init();
#endif
  /* date/time menu initialisation */
  menu_dt_coldstart_init();

  /* advent menu initialisation */
  menu_adv_coldstart_init();

  /* reboots read menu initialisation */
  menu_dt_coldstart_init();

  /* miscellaneous menu initialisation */
  menu_misc_coldstart_init();
}


/* react if a key on the HMI was pressed and dtermine the menu page */
void  menu_srv  (void)
{
  uint8_t desired_menu = MENU_INDEX_MAIN;

  if(     (rl_data.key.bank[KP_INDEX].pressed)
     || ( (rl_data.key.bank[KP_INDEX].released) && (rl_data.menu.index != MENU_INDEX_MAX) )
    )
  {
    if( rl_data.menu.index == MENU_INDEX_MAX )
    {
      /* we bring something to the black screen first time - switch on the backlight */
      lcd_backlight_on();

      if( rl_data.key.bank[KP_INDEX].pressed & KP_CURSOR_MIDDLE )
      {
        desired_menu = MENU_INDEX_CURR_VALUE;
      }
    }
    else
    {
      desired_menu = rl_data.menu.index;
    }

    menu_selection(desired_menu);
  }
  else if(rl_data.menu.curr_value.trigger != 0)
  {
    rl_data.menu.curr_value.trigger = 0;

    /* display is showing current values - refresh menu by cyclic timer source */
    {
      if(rl_data.menu.index == MENU_INDEX_CURR_VALUE)
      {
        menu_curr_value_srv();
      }
    }
  }
}

/* if a menu page was selected consider its 1st call and call its init function otherwise the regular service function of that menu page */
void menu_selection  (uint8_t menu_new)
{
  if(rl_data.menu.index != menu_new)
  {
    /* change of menu - init the new menu do not allow further button evaluation */
    rl_data.menu.index = menu_new;

    VoidFnct fct = (VoidFnct)pgm_read_word(&menu_init_fct[rl_data.menu.index]);
    fct();
  }
  else
  {
    VoidFnct fct = (VoidFnct)pgm_read_word(&menu_srv_fct[rl_data.menu.index]);
    fct();
  }
}

/* determine if a changed variable is still within its boundary otherwise trim it */
uint8_t menu_trim_var (uint8_t index, uint8_t value)
{
  uint8_t ret_value;
  uint8_t lower = pgm_read_byte(&menu_var_boundaries[index].lower);
  uint8_t upper = pgm_read_byte(&menu_var_boundaries[index].upper);

  if( (value > upper) &&  (value != 255) )
  {
    ret_value = lower;
  }
  else  if( (value < lower) || (value == 255) )
  {
    ret_value = upper;
  }
  else
  {
    ret_value = value;
  }

  return(ret_value);
}

/* write a viariable in its own way (string, boolean, special format like temperature etc. */
void menu_item_write (uint8_t value, uint8_t item)
{
    uint8_t x         = rl_data.menu.item_stack[item].x;
    uint8_t y         = rl_data.menu.item_stack[item].y;
    uint8_t item_type = rl_data.menu.item_stack[item].item_type;

    if(item_type == MENU_VAR_WEEKDAY)
    {
      lcd_puts_p(x, y, &menu_item_weekday[value][0]);
    }
    else if(item_type == MENU_VAR_BOOLEAN)
    {
      lcd_puts(x, y, (value==0) ? "nein" : "ja  ");
    }
    else if(item_type == MENU_VAR_BOOLEAN_02)
    {
      lcd_putc(x, y, (value==0) ? ' ' : LCD_CHAR_HAKEN);
    }
    else if( (item_type == MENU_VAR_RL_CNT) ||(item_type == MENU_VAR_RL_CNT_FULL) )
    {
      lcd_puts_p(x, y, &menu_room_name[value][0]);
    }
    else if(item_type == MENU_VAR_RC_SYS)
    {
      if(value == 0)
      {
        lcd_putc(x, y, '0');
      }
      else if(value == 1)
      {
        lcd_putc(x, y, 'F');
      }
      else if(value == 2)
      {
        lcd_putc(x, y, '1');
      }
    }
    else if(item_type == MENU_VAR_RC_DEV)
    {
      lcd_putc(x, y, ('A'+value));
    }
    else if(item_type == MENU_VAR_RC_CMD)
    {
        switch(value & RC_DIGIT_VALUE_ON_OFF_MASK)
        {
            case RC_DIGIT_VALUE_OFF:
            {
                lcd_puts_p(x, y, &menu_item_on_off_cmd[0][0]);
                break;
            }
            case RC_DIGIT_VALUE_OFF_FORCE:
            {
                lcd_puts_p(x, y, &menu_item_on_off_cmd[1][0]);
                break;
            }
            case RC_DIGIT_VALUE_ON:
            {
                lcd_puts_p(x, y, &menu_item_on_off_cmd[2][0]);
                break;
            }
            case RC_DIGIT_VALUE_ON_FORCE:
            {
                lcd_puts_p(x, y, &menu_item_on_off_cmd[3][0]);
                break;
            }
            case RC_DIGIT_VALUE_ON_OFF:
            {
                lcd_puts_p(x, y, &menu_item_on_off_cmd[4][0]);
                break;
            }
        }
    }
    else if(item_type == MENU_VAR_TIMER_PAGE)
    {
      lcd_putc(x, y, ('1'+value));
    }
    else if(item_type == MENU_VAR_SHADOWING_SM)
    {
      uint8_t tmp_value;

      if(value == ALM_M_AUTO_MODE_OFF)
      {
        tmp_value = 'o';
      }
      else if(value == ALM_M_AUTO_MODE_INITIALIZED)
      {
        tmp_value = 'i';
      }
      else if(value == ALM_M_AUTO_MODE_DOWNDRIVEN)
      {
        tmp_value = 'd';
      }
      else if(value == ALM_M_AUTO_MODE_UPDRIVEN)
      {
        tmp_value = 'u';
      }
      else if( (value == ALM_M_AUTO_MODE_DRIVING_TO_HIDDEN_SENSOR) || (value == ALM_M_AUTO_MODE_DRIVING_TO_UNHIDDEN_SENSOR) )
      {
        tmp_value = 'm';
      }
      else
      {
        tmp_value = '?';
      }

      lcd_putc(x, y, tmp_value);
    }
    else if(item_type == MENU_VAR_SENSOR)
    {
      lcd_put_three_digit_dec_u8(x, y, value);
    }
    else
    {
      lcd_put_two_digit_dec_u8(x, y, value);
    }

    lcd_gotoxy(x, y);
}

/* increment/decrement a menu item by #1 if cursor left/right was pressed (a menu item describes the position of a variable within a menu page) */
uint8_t menu_srv_dec_inc_item (uint8_t * current_item, uint8_t item_max)
{
    if(rl_data.key.bank[KP_INDEX].pressed == KP_CURSOR_LEFT)
    {
        if(--*current_item == 255)
        {
            *current_item = item_max-1;
        }
    }
    else
    {
        if(++*current_item == item_max)
        {
            /* if cursor is at a binary show the associated room */
            *current_item = 0;
        }
    }

    return (*current_item);
}

/* increment/decrement a vaiable by #1 if cursor up/down is pressed or by #10 id additionally keypad left is longpressed */
uint8_t menu_srv_dec_inc_value (uint8_t item_type, uint8_t * value)
{
    uint8_t tmp_value = * value;

    if(rl_data.key.bank[KP_INDEX].pressed == KP_CURSOR_UP)
    {
        if(rl_data.key.bank[KP_INDEX].longpressed == KP_LEFT)
        {
            tmp_value+=10;
        }
        else
        {
            tmp_value++;
        }
   }
   else
   {
       if(rl_data.key.bank[KP_INDEX].longpressed == KP_LEFT)
       {
           tmp_value-=10;
       }
       else
       {
           tmp_value--;
       }
   }

  tmp_value = * value = menu_trim_var (item_type, tmp_value);

  return (tmp_value);
}


void menu_setup_item_stack(uint8_t * PROGMEM src, uint8_t len)
{
    uint8_t i;
    uint8_t *dst = (uint8_t *) &rl_data.menu.item_stack[0];

    for(i=0; i<len*sizeof(MENU_ITEM); i++)
    {
      *(dst+i) = pgm_read_byte(src+i);
    }
}


#if 0
/* increment/decrement a 16bit-vaiable by #1 if cursor up/down is pressed or by #10 id additionally keypad left is longpressed */
uint16_t menu_srv_dec_inc_value_u16 (uint16_t value)
{
  if(rl_data.key.bank[KP_INDEX].pressed == KP_CURSOR_UP)
  {
     if(rl_data.key.bank[KP_INDEX].longpressed == KP_LEFT)
     {
       value+=10;
     }
     else
     {
       value++;
     }
  }
  else
  {
     if(rl_data.key.bank[KP_INDEX].longpressed == KP_LEFT)
     {
       value-=10;
     }
     else
     {
       value--;
     }
  }

  return (value);
}
#endif
/************************************************************************/
/* copyright: BBC-Hoerer 2016-2018                                      */
/************************************************************************/
