/******************************************************************************/
/* Project:   blind control set                                               */
/* Author:    BBC-Hoerer                                                      */
/*            You may use or modify this code for your purpose.               */
/*            BBC-Hoerer is not responsible for consequences using this code. */
/******************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "i2cmaster.h"
#include "lcd.h"
#include "_rl_com.h"

/* menu treating "motor setup" menu page */
const  PROGMEM char menu_motor_line_1[] = "max. Laufzeit:     s";
const  PROGMEM char menu_motor_line_3[] = "Remote-Code:";

#define MENU_ITEM_RT_MOTOR_MAX 8
const PROGMEM MENU_ITEM menu_item_rt_motor[MENU_ITEM_RT_MOTOR_MAX] =
                                              { {17, 0, MENU_VAR_SECOND},
                                                {13, 2, MENU_VAR_RC_SYS}, {14, 2, MENU_VAR_RC_SYS}, {15, 2, MENU_VAR_RC_SYS}, {16, 2, MENU_VAR_RC_SYS}, {17, 2, MENU_VAR_RC_SYS}, {19, 2, MENU_VAR_RC_DEV},
                                                {0, 3, MENU_VAR_RL_CNT_FULL}
                                              };

#if (MENU_ITEM_RT_MOTOR_MAX > MENU_ITEM_STACK_SIZE)
  #error "item stack too small"
#endif


/* coldstart initialisation of menu page "runtime motor" */
void menu_motor_coldstart_init  (void)
{
  /* start with the first rl */
  rl_data.menu.rt_motor.current_rl=0;
  /* put the cursor to first item */
  rl_data.menu.rt_motor.current_item=0;
}

/* init function of menu page "runtime motor" - called once if menu page has changed */
void menu_motor_init  (void)
{
  lcd_clrscr();

  menu_setup_item_stack( (uint8_t * PROGMEM) &menu_item_rt_motor[0], MENU_ITEM_RT_MOTOR_MAX);

  /* write constant 's' (sekunde) to menu  */
  lcd_puts_p(0, 0,  &menu_motor_line_1[0]);
  lcd_puts_p(0, 2,  &menu_motor_line_3[0]);

  /* rewrite the line showing room and its yes/no (character lines have different lengths) */

  /* write motor runtime */
  menu_item_write(nv_data.motor[rl_data.menu.rt_motor.current_rl].runtime, 0);

  uint8_t sys_code_digit_nr;
  uint8_t digit;
  /* write remote control system- and device code */
  for(sys_code_digit_nr=0; sys_code_digit_nr<5; sys_code_digit_nr++)
  {
    /* write 5 digit system code */
    digit = rc_get_syscode_digit_from_device ( (uint8_t *) &nv_data.motor[rl_data.menu.rt_motor.current_rl].rc_code[0], sys_code_digit_nr);
    menu_item_write(digit, 1+sys_code_digit_nr);
  }

  /* write 1 digit device code */
  digit = rc_get_devcode_digit_from_device ( nv_data.motor[rl_data.menu.rt_motor.current_rl].rc_code[1] );
  menu_item_write(digit, 6);

  /* write room */
  menu_item_write(rl_data.menu.rt_motor.current_rl, MENU_ITEM_RT_MOTOR_MAX-1);

  /* set cursor to the last edited value of alarm menu and switch on cursor */
  lcd_gotoxy(rl_data.menu.item_stack[rl_data.menu.rt_motor.current_item].x, rl_data.menu.item_stack[rl_data.menu.rt_motor.current_item].y);

  lcd_command(LCD_DISP_ON_CURSOR);
}


/* regular service function of menu page "runtime motor" - called on keypad key change */
void menu_motor_srv (void)
{
    uint8_t current_item = rl_data.menu.rt_motor.current_item;
    uint8_t item_type    = rl_data.menu.item_stack[current_item].item_type;

    switch(rl_data.key.bank[KP_INDEX].pressed)
    {
      case KP_CURSOR_LEFT:
      case KP_CURSOR_RIGHT:
      {
        current_item = menu_srv_dec_inc_item ((uint8_t *) & rl_data.menu.rt_motor.current_item, MENU_ITEM_RT_MOTOR_MAX);

        lcd_gotoxy(rl_data.menu.item_stack[current_item].x, rl_data.menu.item_stack[current_item].y);

        break;
      }

      case KP_CURSOR_UP:
      case KP_CURSOR_DOWN:
      {
        if(item_type == MENU_VAR_SECOND)
        {
          uint8_t value;

          value = menu_srv_dec_inc_value(item_type, (uint8_t *) &nv_data.motor[rl_data.menu.rt_motor.current_rl].runtime);

          menu_item_write(value, current_item);
        }
        else if( item_type == MENU_VAR_RC_SYS )
        {
          uint8_t value;

          value = rc_get_syscode_digit_from_device ((uint8_t *) &nv_data.motor[rl_data.menu.rt_motor.current_rl].rc_code[0], current_item-1);
          value = menu_srv_dec_inc_value(item_type, (uint8_t *) & value);
          rc_set_syscode_digit_to_device ((uint8_t *) &nv_data.motor[rl_data.menu.rt_motor.current_rl].rc_code[0], current_item-1, value);
          menu_item_write(value, current_item);
        }
        else if(item_type == MENU_VAR_RC_DEV)
        {
          uint8_t value;

          value = rc_get_devcode_digit_from_device (nv_data.motor[rl_data.menu.rt_motor.current_rl].rc_code[1]);
          value = menu_srv_dec_inc_value(item_type, (uint8_t *) & value);
          rc_set_devcode_digit_to_device ( (uint8_t *) & nv_data.motor[rl_data.menu.rt_motor.current_rl].rc_code[1], value);
          menu_item_write(value, current_item);
        }
        else if(item_type == MENU_VAR_RL_CNT_FULL)
        {
          menu_srv_dec_inc_value(item_type, (uint8_t *) & rl_data.menu.rt_motor.current_rl);

          menu_motor_init();
        }

        break;
      }

      case KP_CURSOR_MIDDLE:
      {
        /* save var and leave menu */
        eeprom_write_var_to_eeprom(EEPROM_ADDRESS(motor[0]), (uint8_t *) &nv_data.motor[0], RL_COUNT_FULL * sizeof(NV_STRUC_MOTOR));

        //menu_selection(MENU_INDEX_MAIN);

        break;
      }

      case KP_RIGHT:
      {
        /* reload var from eeprom and leave menu */
        eeprom_read_var_from_eeprom((uint8_t *) &nv_data.motor[0], EEPROM_ADDRESS(motor[0]), RL_COUNT_FULL * sizeof(NV_STRUC_MOTOR));

        menu_selection(MENU_INDEX_MAIN);

        break;
      }
  }
}

/************************************************************************/
/* copyright: BBC-Hoerer 2016-2018                                      */
/************************************************************************/
