/******************************************************************************/
/* Project:   blind control set                                               */
/* Author:    BBC-Hoerer                                                      */
/*            You may use or modify this code for your purpose.               */
/*            BBC-Hoerer is not responsible for consequences using this code. */
/******************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "i2cmaster.h"
#include "lcd.h"
#include "_rl_com.h"

/* menu treating "night blind" menu page */
/* the year is devided into 24 pages: day 1..14, day 14..end of month + an additional christmas page November, 27th till January, 6th that describing different times for windows with an light bow */

const  PROGMEM char menu_n_blind_line_1[] = "Zeitraum Auf   Ab";
const  PROGMEM char menu_n_blind_line_2[] = "  .  .-    :     :";

#define MENU_ITEM_N_BLIND_MAX 7
const PROGMEM MENU_ITEM menu_item_n_blind[MENU_ITEM_N_BLIND_MAX] =
                                            { {0, 1, MENU_VAR_N_BLIND_PAGE}, {9, 1, MENU_VAR_HOUR}, {12, 1, MENU_VAR_MINUTE}, {15, 1, MENU_VAR_HOUR}, {18, 1, MENU_VAR_MINUTE},
                                              {0, 3, MENU_VAR_RL_CNT_FULL}, {15, 3, MENU_VAR_BOOLEAN}
                                            };

#if (MENU_ITEM_N_BLIND_MAX > MENU_ITEM_STACK_SIZE)
  #error "item stack too small"
#endif


/* coldstart initialisation of menu page "night blind" */
void menu_n_blind_coldstart_init  (void)
{
  /* start with the first rl */
  rl_data.menu.n_blind.current_rl=0;
  /* put the cursor to MENU_ITEM_N_BLIND_FIRST_EDITABLE (hour_up) */
  rl_data.menu.n_blind.current_item=0;

  rl_data.menu.n_blind.current_page = rl_data.alm.current_page;
}

/* rewrite the room and its yes/no decision */
static void menu_n_blind_write_room_yes_no_decision  (void)
{
  /* write room */
  menu_item_write(rl_data.menu.n_blind.current_rl, 5);

  /* write rooms yes/no decision */
  menu_item_write((nv_data.n_blind_window.included & _BV(rl_data.menu.n_blind.current_rl)) ? 1 : 0, 6);

  /* set cursor to the last edited value of alarm menu and switch on cursor */
  lcd_gotoxy(rl_data.menu.item_stack[rl_data.menu.n_blind.current_item].x, rl_data.menu.item_stack[rl_data.menu.n_blind.current_item].y);
}


/* init function of menu page "night blind" - called once if menu page has changed */
void menu_n_blind_init	(void)
{
  uint8_t page = rl_data.menu.n_blind.current_page;
  uint8_t days, month;

  lcd_clrscr();

  menu_setup_item_stack( (uint8_t * PROGMEM) &menu_item_n_blind[0], MENU_ITEM_N_BLIND_MAX);

  /* write 1. line of alarm menu */
  lcd_puts_p(0, 0,  &menu_n_blind_line_1[0]);

  /* generate days and month to be written as time period */
  n_blind_page_to_day_month__(page, days, month);

  /* write 2. line of alam menu  */
  lcd_puts_p(0, 1,  &menu_n_blind_line_2[0]);
  lcd_put_two_digit_dec_u8(0, 1, days);
  lcd_put_two_digit_dec_u8(3, 1, month);

  /* write editable parameters of 2. line of alamr menu (up/down times of blinds) */
  menu_item_write(nv_data.n_blind[page].hour_up,     1);
  menu_item_write(nv_data.n_blind[page].minute_up,   2);
  menu_item_write(nv_data.n_blind[page].hour_down,   3);
  menu_item_write(nv_data.n_blind[page].minute_down, 4);

  menu_n_blind_write_room_yes_no_decision();

  lcd_command(LCD_DISP_ON_CURSOR);
}


/* regular service function of menu page "night blind" - called on keypad key change */
void menu_n_blind_srv (void)
{
  uint8_t current_item = rl_data.menu.n_blind.current_item;
  uint8_t item_type    = rl_data.menu.item_stack[current_item].item_type;

  switch(rl_data.key.bank[KP_INDEX].pressed)
  {
    case KP_CURSOR_LEFT:
    case KP_CURSOR_RIGHT:
    {
        current_item = menu_srv_dec_inc_item ((uint8_t *) & rl_data.menu.n_blind.current_item, MENU_ITEM_N_BLIND_MAX);

        lcd_gotoxy(rl_data.menu.item_stack[current_item].x, rl_data.menu.item_stack[current_item].y);

        break;
    }

    case KP_CURSOR_UP:
    case KP_CURSOR_DOWN:
    {
        uint8_t value;

        if(item_type == MENU_VAR_N_BLIND_PAGE)
        {
          menu_srv_dec_inc_value(item_type, (uint8_t *) & rl_data.menu.n_blind.current_page);

          /* rewrite the full content of the display as we have changed a page */
          menu_n_blind_init();
        }
        else if(item_type == MENU_VAR_RL_CNT_FULL)
        {
          menu_srv_dec_inc_value(item_type, (uint8_t *) & rl_data.menu.n_blind.current_rl);

          menu_n_blind_write_room_yes_no_decision();
        }
        else if(item_type == MENU_VAR_BOOLEAN)
        {
          /* toggle the true/yes bit of the assigned blind */
          nv_data.n_blind_window.included ^= _BV(rl_data.menu.n_blind.current_rl);

          menu_n_blind_write_room_yes_no_decision();
        }
        else /* MENU_VAR_HOUR, MENU_VAR_MINUTE */
        {
          value = menu_srv_dec_inc_value(item_type, (uint8_t *) &(nv_data.n_blind[rl_data.menu.n_blind.current_page].hour_up)+current_item-1); /* tricky: item #1 is MENU_VAR_HOUR but var it is at index 0 -> subtract #1 */

          /* write the desired item to display */
          menu_item_write(value, current_item);
        }

        break;
    }

    case KP_CURSOR_MIDDLE:
    {
        /* save var */
        eeprom_write_var_to_eeprom(EEPROM_ADDRESS(n_blind[rl_data.menu.n_blind.current_page]), (uint8_t *) &nv_data.n_blind[rl_data.menu.n_blind.current_page], sizeof(NV_STRUC_N_BLIND));
        eeprom_write_var_to_eeprom(EEPROM_ADDRESS(n_blind_window), (uint8_t *) &nv_data.n_blind_window, sizeof(NV_STRUC_N_BLIND_WINDOW));

        //menu_selection(MENU_INDEX_MAIN);

        break;
    }

    case KP_RIGHT:
    {
      /* reload var from eeprom and leave menu */
      eeprom_read_var_from_eeprom((uint8_t *) &nv_data.n_blind[rl_data.menu.n_blind.current_page], EEPROM_ADDRESS(n_blind[rl_data.menu.n_blind.current_page]), sizeof(NV_STRUC_N_BLIND));
      eeprom_read_var_from_eeprom((uint8_t *) &nv_data.n_blind_window, EEPROM_ADDRESS(n_blind_window), sizeof(NV_STRUC_N_BLIND_WINDOW));

      rl_data.menu.n_blind.current_page = rl_data.alm.current_page;

      menu_selection(MENU_INDEX_MAIN);

      break;
    }
  }
}


/************************************************************************/
/* copyright: BBC-Hoerer 2016-2018                                      */
/************************************************************************/
