/******************************************************************************/
/* Project:   blind control set                                               */
/* Author:    BBC-Hoerer                                                      */
/*            You may use or modify this code for your purpose.               */
/*            BBC-Hoerer is not responsible for consequences using this code. */
/******************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "i2cmaster.h"
#include "lcd.h"
#include "_rl_com.h"

/* motor driver functionality supporting the serial connection of motor relays "PWR/DIR" or the parallel connection of moto relays ""UP/DOWN" */
#if defined MOTOR_PORT_IO

  #define MOTOR_PWR_UP_OUT(_relay)      PORTD = (_relay)
  #define MOTOR_DIR_DOWN_OUT(_relay)   {PORTB = ((_relay) & ~0x04); if ((_relay) & 0x04)  PORTC |= 0x80; else PORTC &= ~0x80;  }

#elif defined MOTOR_PORT_EXPANDER

  #define MOTOR_PWR_UP_OUT(_pwr_relay)     mcp23017_out(MCP23017_ADDRESS_MOTOR+MCP23017_MOTOR_PWR_BANK, (_pwr_relay))
  #define MOTOR_DIR_DOWN_OUT(_dir_relay)   mcp23017_out(MCP23017_ADDRESS_MOTOR+MCP23017_MOTOR_DIR_BANK, (_dir_relay))

#endif


#define MOTOR_TIME_RELAY       125          /* base 1ms: ensure 100ms holding time for a state */

/* init routine of motor module */
void  motor_init  (void)
{
  uint8_t motor_cnt;

  //rl_data.motor.pwr_relay =
  //rl_data.motor.dir_relay = 0;

  #if defined CPU_ATMEGA16
    DDRD  = 0xFF;       /* all PORTD are output */
    DDRB  = 0xFB;       /* beside of PB2 all PORTB are output */
    DDRC |= 0x80;       /* PC7 is output */

  #elif defined CPU_ATMEGA168

    //  set addressing style
    mcp23017_write_register(MCP23017_ADDRESS_MOTOR+MCP23017_MOTOR_DIR_BANK+MCP23017_IOCON, 0x80);

    _delay_ms(5);                  // delay for rearranging register addresses

    /* give a 5 volts reference for whatever? */
    mcp23017_out(MCP23017_ADDRESS_MOTOR+MCP23017_MOTOR_DIR_BANK, 0xff);
    mcp23017_out(MCP23017_ADDRESS_MOTOR+MCP23017_MOTOR_PWR_BANK, 0xff);

    mcp23017_write_register(MCP23017_ADDRESS_MOTOR+MCP23017_MOTOR_DIR_BANK+MCP23017_IODIR, 0);
    mcp23017_write_register(MCP23017_ADDRESS_MOTOR+MCP23017_MOTOR_PWR_BANK+MCP23017_IODIR, 0);

  #endif

  MOTOR_PWR_UP_OUT(0);
  MOTOR_DIR_DOWN_OUT(0);

  for(motor_cnt=0; motor_cnt<RL_COUNT; motor_cnt++)
  {
    rl_data.motor.current_runtime[motor_cnt]=0;
  }
}

/* this function is called in the timer interrupt every second - it checks if the desired maximum runtime of a certain motor has expired and triggers the main loop */
void  motor_update_driving_time  (void)
{
  uint8_t motor_cnt;

  for(motor_cnt=0; motor_cnt<RL_COUNT; motor_cnt++)
  {
    if(rl_data.motor.current_runtime[motor_cnt] != 0 )
    {
      rl_data.motor.current_runtime[motor_cnt]--;

      if(rl_data.motor.current_runtime[motor_cnt] == 0)
      {
        rl_data.motor.perform_stop_req = 1;
      }
    }
  }

  rl_data.motor.service_runtime++;
}

#if defined MOTOR_MODE_DIR_PWR

/* servicing a motor request in serial mode  "POWER/DIR" */
uint8_t  motor_request  (MOTOR_REQ * req)
{
  uint8_t ret_value = 1;

  static uint8_t dir_relay = 0;
  static uint8_t pwr_relay = 0;

  uint8_t tmp_state, rl_number;
  rl_number = req->rl_number;

  {
    uint8_t dir, pwr;
    dir = (dir_relay>>rl_number) & 0x01;
    pwr = (pwr_relay>>rl_number) & 0x01;
    tmp_state = ( (dir<<MOTOR_STATE_SHIFT_DIR) | (pwr<<MOTOR_STATE_SHIFT_PWR) );
  }

  if(req->runtime >= MENU_RT_MOTOR_VALUE_CNT)
  {
    /* user defined runtime */
    rl_data.motor.current_runtime[rl_number] = req->runtime;
  }
  else
  {
    /* setup predefined runtime: MOTOR_RUNTIME_FULL or RT_MOTOR_TIME_SHADOWING */
    rl_data.motor.current_runtime[rl_number] = rl_data.motor.values[rl_number].rt[req->runtime];
  }

  if(req->opc == tmp_state)
  {
    /* nothing to do */
    ret_value = 0;
  }
  else if(req->opc == MOTOR_OPC_HALT)
  {
    rl_data.motor.current_runtime[rl_number] = 0;

    if(tmp_state & (1<<MOTOR_STATE_SHIFT_PWR))
    {
      /* pwr-relay is switched on: power off the desired motor by one transition */
      pwr_relay = (pwr_relay & ( ~(0x01<<rl_number) ));
      MOTOR_PWR_UP_OUT(pwr_relay);
      _delay_ms(MOTOR_TIME_RELAY);   // relay switching delay
    }

    if(tmp_state & (1<<MOTOR_STATE_SHIFT_DIR))
    {
      /* the direction relay must not stay active due to power consumtion without need  */
      dir_relay = (dir_relay & ( ~(0x01<<rl_number) ));
      MOTOR_DIR_DOWN_OUT(dir_relay);
      _delay_ms(MOTOR_TIME_RELAY);   // relay switching delay
    }

    wb_enter_halt_state(rl_number);
  }
  else
  {
    /* 1. step: switch off power if needed */
    if(tmp_state & (1<<MOTOR_STATE_SHIFT_PWR))
    {
      /* pwr-relay is switched on: power off the desired motor by one transition */
      pwr_relay = (pwr_relay & ( ~(0x01<<rl_number) ));
      MOTOR_PWR_UP_OUT(pwr_relay);
      _delay_ms(MOTOR_TIME_RELAY);   // relay switching delay
    }

    /* 2. step: change direction if needed */
    if( (req->opc & (1<<MOTOR_STATE_SHIFT_DIR)) ^ (tmp_state & (1<<MOTOR_STATE_SHIFT_DIR)) )
    {
      /* erase direction bit */
      dir_relay = (dir_relay & ( ~(0x01<<rl_number) ));

      if /* set direction bit due to req->opc */
         (req->opc & (1<<MOTOR_STATE_SHIFT_DIR))
      {
        dir_relay = (dir_relay | ( 0x01<<rl_number ));
      }

      MOTOR_DIR_DOWN_OUT(dir_relay);
      _delay_ms(MOTOR_TIME_RELAY);   // relay switching delay
    }

    /* 3. step: switch on power (unconditioned) */
    {
      /* pwr-relais is switched on: power off the desired motor by one transition */
      pwr_relay = (pwr_relay | ( (0x01<<rl_number) ));
      MOTOR_PWR_UP_OUT(pwr_relay);
      _delay_ms(MOTOR_TIME_RELAY);   // relay switching delay
    }
  }

  return(ret_value);
}

#else /* MOTOR_MODE_UP_DOWN, MOTOR_MODE_UP_DOWN_JL_SETUP */

/* servicing a motor request in parallel mode  "UP/DOWN" */
uint8_t  motor_request  (MOTOR_REQ * req)
{
  uint8_t ret_value = 1;
  static uint8_t up_relay = 0;
  static uint8_t down_relay = 0;

  uint8_t tmp_state, rl_number;
  rl_number = req->rl_number;

  {
    uint8_t up, down;
    up   = (up_relay  >>rl_number) & 0x01;
    down = (down_relay>>rl_number) & 0x01;
    tmp_state = ( (up<<MOTOR_STATE_SHIFT_UP) | (down<<MOTOR_STATE_SHIFT_DOWN) );
  }

  if(req->runtime == MOTOR_RUNTIME_FULL)
  {
    /* setup predefined runtime: MOTOR_RUNTIME_FULL */
    rl_data.motor.current_runtime[rl_number] = nv_data.motor[rl_number].runtime;
    if(req->opc == MOTOR_OPC_UP)
    {
      /* give additional seconds when driving up - this lasts a little bit longer */
      rl_data.motor.current_runtime[rl_number] += 2;
    }
  }
  else
  {
    /* user defined runtime */
    rl_data.motor.current_runtime[rl_number] = req->runtime;
  }

  if(req->opc == tmp_state)
  {
    /* nothing to do */
    ret_value = 0;
  }
  else if(req->opc == MOTOR_OPC_HALT)
  {
    rl_data.motor.current_runtime[rl_number] = 0;

    up_relay = (up_relay & ( ~(0x01<<rl_number) ));
    MOTOR_PWR_UP_OUT(up_relay);
    down_relay = (down_relay & ( ~(0x01<<rl_number) ));
    MOTOR_DIR_DOWN_OUT(down_relay);
    _delay_ms(MOTOR_TIME_RELAY);   // relay switching delay

    wb_enter_halt_state(rl_number);
  }
  else if( (req->opc == MOTOR_OPC_PGM) && (tmp_state == MOTOR_OPC_HALT) )
  {
    /* Jarolift programming limit stop */
    up_relay   = (up_relay |   ( (0x01<<rl_number) ));
    MOTOR_PWR_UP_OUT(up_relay);
    down_relay = (down_relay | ( (0x01<<rl_number) ));
    MOTOR_DIR_DOWN_OUT(down_relay);
    _delay_ms(MOTOR_TIME_RELAY);   // relay switching delay
  }
  else
  {
    if( tmp_state != MOTOR_OPC_HALT ) /* direct turn from up to down or vice versa: stop first and await the switching delay */
    {
      up_relay = (up_relay & ( ~(0x01<<rl_number) ));
      MOTOR_PWR_UP_OUT(up_relay);
      down_relay = (down_relay & ( ~(0x01<<rl_number) ));
      MOTOR_DIR_DOWN_OUT(down_relay);
      _delay_ms(MOTOR_TIME_RELAY);   // relay switching delay
    }
    if(req->opc == MOTOR_OPC_UP)
    {
      up_relay   = (up_relay |   ( (0x01<<rl_number) ));
      MOTOR_PWR_UP_OUT(up_relay);
      _delay_ms(MOTOR_TIME_RELAY);   // relay switching delay
    }
    else if(req->opc == MOTOR_OPC_DOWN)
    {
      down_relay   = (down_relay |   ( (0x01<<rl_number) ));
      MOTOR_DIR_DOWN_OUT(down_relay);
      _delay_ms(MOTOR_TIME_RELAY);   // relay switching delay
    }
  }

  return(ret_value);
}

#endif

/* this function is called in the main loop. If the maximum runtime has expired it stops the associated motor by a halt request */
/* moreover it is checked and performed if due if a queued request of alm_manager n_blind is to be executed (sequential service of night blinds) - not more do all requests with a 2sec. delay..*/
void  motor_check_new_request          (void)
{
  /* check if the runtime of a motor has been reached -> halt motor */
  MOTOR_REQ req;

  uint8_t motor_cnt, tmp_motor_stop;

  tmp_motor_stop = 0;

  cli();

  if(rl_data.motor.perform_stop_req != 0)
  {
    rl_data.motor.perform_stop_req = 0;

    tmp_motor_stop = 1;
  }

  sei();

  if(tmp_motor_stop != 0)
  {
    for(motor_cnt=0; motor_cnt<RL_COUNT; motor_cnt++)
    {
      if(rl_data.motor.current_runtime[motor_cnt] == 0 )
      {
        req.opc = MOTOR_OPC_HALT;

        /* ignore sensor when day blind shall drive up */
        req.rl_number   = motor_cnt;
        req.runtime     = MOTOR_RUNTIME_FULL;

        motor_request(&req);
      }
    }

    //alm_manager_perform_n_blind_request_if_due();
  }

  alm_manager_perform_d_blind_request_callback();
}


void  motor_service_rc_telegram (void)
{
    if(nv_data.misc.properties & RL_PROP_SUPPORT_RC_RCVE)
    {
        uint8_t rl_count;

        for(rl_count=0; rl_count<RL_COUNT; rl_count++)
        {
            /* seek for fitting remote control key */
            if(    (rc_get_devcode_digit_from_device(nv_data.motor[rl_count].rc_code[1]) == rl_data.rc_rx.user_tel.device_code)
                && (nv_data.motor[rl_count].rc_code[0] == rl_data.rc_rx.user_tel.raw[0])
                && ( (nv_data.motor[rl_count].rc_code[1] & 0xC0) == (rl_data.rc_rx.user_tel.raw[1] & 0xC0) )
              )
              {
                  /* found fitting remote control key */

                  rl_data.rc_rx.timeout = RC_KEY_TIMEOUT;            /* retrigger timeout value */

                  if(rl_data.rc_rx.user_tel.on_off_code == RC_DIGIT_VALUE_ON)
                  {
                    rl_data.rc_rx.key[WB_UP_INDEX] = _BV(rl_count); /* mark the key bit of the blind wallbutton bank */
                  }
                  else if(rl_data.rc_rx.user_tel.on_off_code == RC_DIGIT_VALUE_OFF)
                  {
                    rl_data.rc_rx.key[WB_DOWN_INDEX] = _BV(rl_count); /* mark the key bit of the blind wallbutton bank */
                  }
                  else if(rl_data.rc_rx.user_tel.on_off_code == RC_DIGIT_VALUE_ON_OFF)
                  {
                    rl_data.alm.n_blind_tmp_exclude_bitlane |= _BV(rl_count); /* mark the key bit of the blind wallbutton bank */
                  }

                  break;
              }
        }
    }
}

/************************************************************************/
/* copyright: BBC-Hoerer 2016-2018                                      */
/************************************************************************/

