/******************************************************************************/
/* Project:   blind control set                                               */
/* Author:    BBC-Hoerer                                                      */
/*            You may use or modify this code for your purpose.               */
/*            BBC-Hoerer is not responsible for consequences using this code. */
/******************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "i2cmaster.h"
#include "lcd.h"
#include "_rl_com.h"

/* this module treats reception of a remote control telegram with on-off-keying (hs2262 chip ) */

#if defined CPU_ATMEGA16

  #define   RC_RX_BIT_HIGH         ( PINB & (1<<PINB2) )

#elif defined CPU_ATMEGA168

  #define   RC_RX_BIT_HIGH         ( PIND & (1<<PIND2) )

#endif

enum
{
  RC_RX_STATE_AWAIT_SILENCE,
  RC_RX_STATE_AWAIT_TIMER,
  RC_RX_STATE_AWAIT_FLANC,
  RC_RX_STATE_PREPARE_END_OF_FRAME,
  RC_RX_STATE_AWAIT_END_OF_FRAME,
  RC_RX_STATE_IDLE,
} RC_RX_STATE;

#define rc_rx_disable_receive__()                         \
{                                                         \
    GICR = 0x00;    /* disable external interrupt #2 */   \
}

#define rc_rx_enable_receive__()                          \
{                                                         \
    GIFR = 0x20;    /* clear possibly interrupt #2 */     \
    GICR = 0x20;    /* enable external interrupt #2 */    \
}



static uint8_t rc_rx_check_devcode_digit_in_telegram (uint8_t dev_code_digit_nr)
{
  /* check if the received telgram from the remote control contains the given device code digti number (return 1 in this case, otherwise 0) */
  uint8_t byte_loc   = rc_dev_code[dev_code_digit_nr].byte_loc;
  uint8_t byte_shift = rc_dev_code[dev_code_digit_nr].byte_shift;

  if /* the assigned device code is coded with dip-switch "ON" that means that returns the 0, all other device code positions are set to floating */
     ( ((rl_data.rc_rx.user_tel.raw[byte_loc] >> byte_shift) & RC_DIGIT_VALUE_MASK ) == RC_DIGIT_VALUE_ZERO )
  {
    return(1);
  }
  else
  {
    return(0);
  }
}


static void rc_rx_start_timer  (uint8_t _time)
{
#if defined CPU_ATMEGA16
  TCCR0  = 0;
  OCR0   = (_time);   /* timer compare interrupt after each 4us* _time */
  TCNT0  = 0;
  TCCR0  = 0b00001011;  /* CTC OCR is top value, prescaler 64; CPU-Clock -> 4us base */
#elif defined CPU_ATMEGA168
  TCCR0B  = 0;         /* stop timer */
  OCR0A   = (_time);   /* timer compare interrupt after each 4us* _time */
  TCNT0   = 0;
  TCCR0A  = 0b00000010; /* CTC OCRA is top value */
  TCCR0B  = 0b00000011; /* prescaler 64; CPU-Clock -> 4us base */
#endif
}

static void rc_rx_prepare_ooc_reception  (void)
{
  rl_data.rc_rx.time_multiplier = 12;

  rc_rx_start_timer(RC_TIME_2_TACT); /* wait two tacts */

  rl_data.rc_rx.state = RC_RX_STATE_AWAIT_SILENCE;
}


static void rc_rx_stop_timer (void)
{
#if defined CPU_ATMEGA16
  TCCR0 = 0;
#elif defined CPU_ATMEGA168
  TCCR0B  = 0;
#endif

}


#if defined CPU_ATMEGA16
  ISR(INT2_vect)
#elif defined CPU_ATMEGA168
  ISR(INT0_vect)
#endif
{
  rc_rx_disable_receive__();

  /* we received a l/h flanc - after two tacts we decide if we received a manchester coded #0 or #1 */
  rc_rx_start_timer(RC_TIME_2_TACT); /* wait two tacts */

  if /* we have been interrupted while awaiting silence */
    (rl_data.rc_rx.state == RC_RX_STATE_AWAIT_SILENCE)
  {
    rl_data.rc_rx.time_multiplier = 12;
  }
  else if(rl_data.rc_rx.state == RC_RX_STATE_AWAIT_FLANC)
  {
    /* ok */
    rl_data.rc_rx.state = RC_RX_STATE_AWAIT_TIMER;
  }
  else if(rl_data.rc_rx.state == RC_RX_STATE_PREPARE_END_OF_FRAME)
  {
    /* end of frame starts with a flanc -> from there await 1 tact #1 and 31 tacts silence */
    rl_data.rc_rx.time_multiplier = 12;

    rl_data.rc_rx.state = RC_RX_STATE_AWAIT_END_OF_FRAME;
  }
  else if(rl_data.rc_rx.state == RC_RX_STATE_AWAIT_TIMER)
  {
    /* next 2 flancs arrived earlier than expected -> error */
    rc_rx_prepare_ooc_reception();
  }
  else if(rl_data.rc_rx.state == RC_RX_STATE_AWAIT_END_OF_FRAME)
  {
    /* next 2 flancs arrived earlier than expected -> error */
    rc_rx_prepare_ooc_reception();
  }
}

#if defined CPU_ATMEGA16
  ISR(TIMER0_COMP_vect)
#elif defined CPU_ATMEGA168
  ISR(TIMER0_COMPA_vect)
#endif
{
  rc_rx_stop_timer();

  rc_rx_enable_receive__();

  if(rl_data.rc_rx.state == RC_RX_STATE_AWAIT_SILENCE)
  {
    if(--rl_data.rc_rx.time_multiplier == 0)
    {
      rl_data.rc_rx.byte_location = 0;
      rl_data.rc_rx.bit_location  = 7;

      rl_data.rc_rx.curr_tel_raw[0] = rl_data.rc_rx.curr_tel_raw[1] = rl_data.rc_rx.curr_tel_raw[2] = 0;

      rl_data.rc_rx.state = RC_RX_STATE_AWAIT_FLANC;
    }
    else
    {
      rc_rx_start_timer(RC_TIME_2_TACT); /* wait two tacts */
    }
  }
  else if( rl_data.rc_rx.state == RC_RX_STATE_AWAIT_FLANC )
  {
    /* timeout */
    rc_rx_prepare_ooc_reception();
  }
  if( rl_data.rc_rx.state == RC_RX_STATE_AWAIT_TIMER )
  {
    rl_data.rc_rx.state = RC_RX_STATE_AWAIT_FLANC;

    if( RC_RX_BIT_HIGH )
    {
      rl_data.rc_rx.curr_tel_raw[rl_data.rc_rx.byte_location] |= 1<<rl_data.rc_rx.bit_location;
    }

    if(--rl_data.rc_rx.bit_location == 0xFF)
    {
      rl_data.rc_rx.bit_location = 7;

      if(++rl_data.rc_rx.byte_location == 3)
      {
        /* await sync */
        rl_data.rc_rx.state = RC_RX_STATE_PREPARE_END_OF_FRAME;
      }
    }

    rc_rx_start_timer(RC_TIME_TIMEOUT); /* wait a timeout -> must not happen */
  }
  else if( rl_data.rc_rx.state == RC_RX_STATE_PREPARE_END_OF_FRAME )
  {
    /* timeout */
    rc_rx_prepare_ooc_reception();
  }
  else if( rl_data.rc_rx.state == RC_RX_STATE_AWAIT_END_OF_FRAME )
  {
    if(rl_data.rc_rx.time_multiplier != 0)
    {
      --rl_data.rc_rx.time_multiplier;

      rc_rx_start_timer(RC_TIME_2_TACT); /* wait two tacts */
    }
    else
    {
      /* telegram received */
      rc_rx_stop_timer();

      if(    (rl_data.rc_rx.last_tel_raw[0] == rl_data.rc_rx.curr_tel_raw[0])
          && (rl_data.rc_rx.last_tel_raw[1] == rl_data.rc_rx.curr_tel_raw[1])
          && (rl_data.rc_rx.last_tel_raw[2] == rl_data.rc_rx.curr_tel_raw[2])
          && (rl_data.rc_rx.curr_tel_raw[2] != 0x5F) /* perhaps error when button is released? - device codes are set to "float" and both "on" and "off" are set to "1" */
        )
      {
          uint8_t tmp_dev_code;

          rl_data.rc_rx.state = RC_RX_STATE_IDLE;

          rl_data.rc_rx.user_tel.raw[0] = rl_data.rc_rx.curr_tel_raw[0];
          rl_data.rc_rx.user_tel.raw[1] = rl_data.rc_rx.curr_tel_raw[1];
          rl_data.rc_rx.user_tel.raw[2] = rl_data.rc_rx.curr_tel_raw[2];

          rl_data.rc_rx.user_tel.device_code = 0xFF; /* preset dev_code to invalid */

          for(tmp_dev_code=0; tmp_dev_code<5;tmp_dev_code++)
          {
            if (rc_rx_check_devcode_digit_in_telegram(tmp_dev_code) != 0)
            {
              rl_data.rc_rx.user_tel.device_code = tmp_dev_code;

              break;
            }
          }

          switch(rl_data.rc_rx.user_tel.raw[2] & RC_DIGIT_VALUE_ON_OFF_MASK)
          {
              case RC_DIGIT_VALUE_ON:
                  { rl_data.rc_rx.user_tel.on_off_code = RC_DIGIT_VALUE_ON; break; }
              case RC_DIGIT_VALUE_ON_FORCE:
                  { rl_data.rc_rx.user_tel.on_off_code = RC_DIGIT_VALUE_ON_FORCE; break; }
              case RC_DIGIT_VALUE_OFF:
                  { rl_data.rc_rx.user_tel.on_off_code = RC_DIGIT_VALUE_OFF; break; }
              case RC_DIGIT_VALUE_OFF_FORCE:
                  { rl_data.rc_rx.user_tel.on_off_code = RC_DIGIT_VALUE_OFF_FORCE; break; }
              case RC_DIGIT_VALUE_ON_OFF:
                  { rl_data.rc_rx.user_tel.on_off_code = RC_DIGIT_VALUE_ON_OFF; break; }
              default:
                  { rl_data.rc_rx.user_tel.on_off_code = RC_DIGIT_VALUE_NO; break; }
          }

          if( (rl_data.rc_rx.user_tel.device_code != 0xFF) && (rl_data.rc_rx.user_tel.on_off_code != RC_DIGIT_VALUE_NO) )
          {
            rl_data.rc_rx.evaluate = 1; /* sign new telegram reception */
          }
          else
          {
            rc_rx_prepare_ooc_reception(); /* telegram was built wrong - prepare new reception */
          }
      }
      else
      {
          rl_data.rc_rx.last_tel_raw[0] = rl_data.rc_rx.curr_tel_raw[0];
          rl_data.rc_rx.last_tel_raw[1] = rl_data.rc_rx.curr_tel_raw[1];
          rl_data.rc_rx.last_tel_raw[2] = rl_data.rc_rx.curr_tel_raw[2];
      }
    }
  }
}


void  rc_rx_init  (void)
{
#if defined CPU_ATMEGA16
  MCUCSR |= 0x40;  /* rising edge causes interrupt */
  GICR = 0x20;    /* enable external interrupt #2 */
  GIFR = 0x20;    /* clear possibly interrupt #2 */
  TIMSK |= 0b00000010;
#elif defined CPU_ATMEGA168
  EICRA  = 0x03;
  EIMSK  = 0x01;
  TIMSK0 = 0b00000010;
#endif
  rl_data.rc_rx.user_tel.on_off_code = RC_DIGIT_VALUE_NO;

  rc_rx_stop_timer();

  rc_rx_prepare_ooc_reception();
}

void  rc_rx_timeout  (void)
{
  if(rl_data.rc_rx.timeout)
  {
    --rl_data.rc_rx.timeout;
  }

  if(rl_data.key.service_mode.time_accept_rc_code_change != 0)
  {
      rl_data.key.service_mode.time_accept_rc_code_change--;
  }
}


void  rc_rx_evaluate  (void)
{
    if( rl_data.rc_rx.timeout == 0 )
    {
      /* on a timeout no key can be pressed - so delete former value */
      rl_data.rc_rx.key[WB_UP_INDEX] = rl_data.rc_rx.key[WB_DOWN_INDEX] = 0;
    }

    if( rl_data.rc_rx.evaluate == 1 )
    {
        rl_data.rc_rx.evaluate = 0;

        if /* telegram was received whilst in service mode -> evaluate blind setup */
          (service_mode_evaluation() == SERVICE_MODE_EVAL_NOT_DONE)
        {
            /* telegram was received whilst not in service mode -> evaluate blind control */
            motor_service_rc_telegram();
        }

        rc_rx_prepare_ooc_reception(); /* done and ready for the next rc telegram */
    }
}


/************************************************************************/
/* copyright: BBC-Hoerer 2016-2018                                      */
/************************************************************************/
