/******************************************************************************/
/* Project:   blind control set                                               */
/* Author:    BBC-Hoerer                                                      */
/*            You may use or modify this code for your purpose.               */
/*            BBC-Hoerer is not responsible for consequences using this code. */
/******************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "i2cmaster.h"
#include "lcd.h"
#include "_rl_com.h"

/* this module transmits telegrams like a remote control telegram with on-off-keying (hs2262 chip ) */

#define RC_TX_TIME_MIN        2     /* 2 * 8us */
#define RC_TX_TIME_1_TACT     46    /* 46 * 8us -> 368us */
#define RC_TX_TIME_3_TACT     139   /* 139 * 8us -> 1112us */


#if defined CPU_ATMEGA16

  #define   RC_TX_SET_BIT_HIGH        ( PORTC = PORTC |  (1<<PORTC5) )
  #define   RC_TX_SET_BIT_LOW         ( PORTC = PORTC & ~(1<<PORTC5) )
  #define   RC_TX_GET_BIT             ( PORTC & (1<<PORTC5) )

#elif defined CPU_ATMEGA168

  #define   RC_TX_SET_BIT_HIGH        ( PORTD = PORTD |  (1<<PORTD6) )
  #define   RC_TX_SET_BIT_LOW         ( PORTD = PORTD & ~(1<<PORTD6) )
  #define   RC_TX_GET_BIT             ( PORTD & (1<<PORTD6) )

#endif

static void rc_tx_start_timer  (uint8_t _time)
{
#if defined CPU_ATMEGA16
  TCCR2  = 0;
  OCR2   = (_time);   /* timer compare interrupt after each 4us* _time */
  TCNT2  = 0;
  TCCR2  = 0b00001101;  /* CTC OCR is top value, prescaler 128; CPU-Clock -> 8us base */
#elif defined CPU_ATMEGA168
  TCCR2B  = 0;         /* stop timer */
  OCR2A   = (_time);   /* timer compare interrupt after each 4us* _time */
  TCNT2   = 0;
  TCCR2A  = 0b00000010; /* CTC OCRA is top value */
  TCCR2B  = 0b00000101; /* prescaler 128; CPU-Clock -> 8us base */
#endif
}

static void rc_tx_stop_timer (void)
{
#if defined CPU_ATMEGA16
  TCCR2 = 0;
#elif defined CPU_ATMEGA168
  TCCR2B  = 0;
#endif

}


static uint8_t rc_tx_prepare_transmission (void)
{
  uint8_t new_time;

  if(--rl_data.rc_tx.repetition)
  {
    /* we have to send a telegram (first time or repetition) */
    /* prepare the telegram scanner starting with first byte and the MSB */
    rl_data.rc_tx.bit_location  = 7;
    rl_data.rc_tx.byte_location = 0;

    /* idle is lof level */
    RC_TX_SET_BIT_LOW;

    /* give a minimum time - after this the timer interrupt appears and forms the first bit */
    new_time = RC_TX_TIME_MIN;
  }
  else
  {
    /* no telegram to issue - sign this with returning #0 - no new timer start */
    new_time = 0;

    /* nothing more to transmit */
    rl_data.rc_tx.busy = 0;
  }

  return(new_time);
}

#if defined CPU_ATMEGA16
  ISR(TIMER2_COMP_vect)
#elif defined CPU_ATMEGA168
  ISR(TIMER2_COMPA_vect)
#endif
{
  uint8_t bit_value, new_time;

  /* at first stop this timer - the following waveforming state machine determines the next timer value */
  rc_tx_stop_timer();

  /* the waveforming statemachine uses rl_data.rc_tx.byte_location describing the state */
  if(rl_data.rc_tx.byte_location < 3)
  {
    /* transmitting telegram */
    bit_value = rl_data.rc_tx.telegram[rl_data.rc_tx.byte_location] & (1<<rl_data.rc_tx.bit_location);

    if(RC_TX_GET_BIT == 0)
    {
      /* l/h transition */
      RC_TX_SET_BIT_HIGH;

      /* transmitting #1 must last 3 tacts, transmitting #0 must last 1 tact */
      new_time = ( (bit_value != 0) ? RC_TX_TIME_3_TACT : RC_TX_TIME_1_TACT );
    }
    else
    {
      /* h/l transition */
      RC_TX_SET_BIT_LOW;

      /* #1 has been transmitted -> wait 1 tact now, #0 has been transmitted -> wait 3 tacts now */
      new_time = ( (bit_value != 0) ? RC_TX_TIME_1_TACT : RC_TX_TIME_3_TACT );

      /* the h/l flanc of a bit has been done - determine the next bit to form by incrementing bit_location/byte_location */
      if(--rl_data.rc_tx.bit_location == 255)
      {
        rl_data.rc_tx.bit_location = 7;

        if(++rl_data.rc_tx.byte_location == 3)
        {
          /* the bits of the telegram are put into three bytes - and they have been sent now -> next step is entering sync */
        }
      }
    }
  }
  else if(rl_data.rc_tx.byte_location == 3)
  {
    /* entering sync start with 1 tact high level */
    RC_TX_SET_BIT_HIGH;

    new_time = RC_TX_TIME_1_TACT;

    rl_data.rc_tx.byte_location++; /* sign state transition entering sync -> sync */
  }
  else if(++rl_data.rc_tx.byte_location < (3+31) )
  {
    /* sync continues with additional 31 tacts low level */
    RC_TX_SET_BIT_LOW;

    new_time = RC_TX_TIME_1_TACT;
  }
  else
  {
    /* telegram and sync transmitted - check if we have to repeat the telegram or if we have finished */
    new_time = rc_tx_prepare_transmission();
  }

  if (new_time != 0)
  {
    /* only if we still have to send we start the timer so that the state machine continues of wave forming */
    rc_tx_start_timer(new_time);
  }
}


void  rc_tx_init  (void)
{
#if defined CPU_ATMEGA16
  TIMSK |= 0b10000000;        /* enable OCIE2 isr */
#elif defined CPU_ATMEGA168
  TIMSK2 = 0b00000010;
#endif

  rc_tx_stop_timer();

  rl_data.rc_tx.repetition =
  rl_data.rc_tx.busy         = 0;
  rl_data.rc_tx.queue.rd_idx =
  rl_data.rc_tx.queue.wr_idx = 0;
}


static void rc_tx_inc_wr_idx(void)
{
    if(++rl_data.rc_tx.queue.wr_idx == RC_TX_QUEUE_SIZE)
    {
      rl_data.rc_tx.queue.wr_idx = 0;
      if(rl_data.rc_tx.queue.wr_idx == rl_data.rc_tx.queue.rd_idx)
      {
        /* too many requests -> fatal, await watchdog */
        for(;;);
      }
    }
}


static void rc_tx_inc_rd_idx(void)
{
    if(++rl_data.rc_tx.queue.rd_idx == RC_TX_QUEUE_SIZE)
    {
      rl_data.rc_tx.queue.rd_idx = 0;
    }
}


void     rc_tx_send_command          (RC_COMMAND * rc_command, uint8_t on_off_command, uint8_t repetition_cnt)
{
    rl_data.rc_tx.queue.req[rl_data.rc_tx.queue.wr_idx].rc_command.syscode     = rc_command->syscode;      /* write syscode digit #1..4 */
    rl_data.rc_tx.queue.req[rl_data.rc_tx.queue.wr_idx].rc_command.sys_devcode = rc_command->sys_devcode;  /* write syscode digit #5    */
    rl_data.rc_tx.queue.req[rl_data.rc_tx.queue.wr_idx].on_off_command         = on_off_command;           /* write on/off in lower nibble - upper nibble automatically cleared */
    rl_data.rc_tx.queue.req[rl_data.rc_tx.queue.wr_idx].repetition_cnt         = repetition_cnt;           /* write on/off in lower nibble - upper nibble automatically cleared */

    rc_tx_inc_wr_idx();
}


void     rc_tx_command_scheduler          (void)
{
    #if defined RC_TEST_TX
        if(rl_data.rc_tx.test_trigger != 0)
        {
            rl_data.rc_tx.test_trigger = 0;

            /* test: toggle on/off an outlet with SysCode 00000 and DevCode D every 2sec.*/
            if((rl_data.time.current_second%6) == 0)
            {
              RC_COMMAND command;
              command.syscode     = (uint8_t) (RC_TEST_TX>>8); /* syscode 1..4                              */
              command.sys_devcode = (uint8_t) (RC_TEST_TX>>0); /* syscode 5 and devcode (binary coded)  */
              rc_tx_send_command(& command, RC_DIGIT_VALUE_ON, 4);
            }
            else if((rl_data.time.current_second%6) == 2)
            {
              RC_COMMAND command;
              command.syscode     = (uint8_t) (RC_TEST_TX>>8); /* syscode 1..4                              */
              command.sys_devcode = (uint8_t) (RC_TEST_TX>>0); /* syscode 5 and devcode (binary coded)  */
              rc_tx_send_command(& command, RC_DIGIT_VALUE_OFF, 4);
            }
        }
    #endif

    if( (rl_data.rc_tx.queue.rd_idx != rl_data.rc_tx.queue.wr_idx) && (rl_data.rc_tx.busy == 0) )
    {
        uint8_t dev_code_digit, tmp_dev_code_digit;
        rl_data.rc_tx.busy = 1;

        /* marshall the syscode and the on/off command of the remote control telegram */
        rl_data.rc_tx.telegram[0] = rl_data.rc_tx.queue.req[rl_data.rc_tx.queue.rd_idx].rc_command.syscode;              /* write syscode digit #1..4 */
        rl_data.rc_tx.telegram[1] = rl_data.rc_tx.queue.req[rl_data.rc_tx.queue.rd_idx].rc_command.sys_devcode & 0xC0;   /* write syscode digit #5    */
        rl_data.rc_tx.telegram[2] = rl_data.rc_tx.queue.req[rl_data.rc_tx.queue.rd_idx].on_off_command;                   /* write on/off in lower nibble - upper nibble automatically cleared */

        /* marshall the devcode of the remote control telegram */
        dev_code_digit = rc_get_devcode_digit_from_device(rl_data.rc_tx.queue.req[rl_data.rc_tx.queue.rd_idx].rc_command.sys_devcode);

        for(tmp_dev_code_digit=0; tmp_dev_code_digit<5; tmp_dev_code_digit++)
        {
            uint8_t byte_loc   = rc_dev_code[tmp_dev_code_digit].byte_loc;
            uint8_t byte_shift = rc_dev_code[tmp_dev_code_digit].byte_shift;

            uint8_t dev_code = (dev_code_digit == tmp_dev_code_digit) ? RC_DIGIT_VALUE_ZERO : RC_DIGIT_VALUE_FLOATING;
            dev_code = dev_code<<byte_shift;
            rl_data.rc_tx.telegram[byte_loc] = rl_data.rc_tx.telegram[byte_loc] | dev_code;
        }

        /* determine the telegram repetition counter - at least 2 consecutive telegrams are neeeded for decoding */
        rl_data.rc_tx.repetition = rl_data.rc_tx.queue.req[rl_data.rc_tx.queue.rd_idx].repetition_cnt;

        rl_data.rc_tx.queue.req[rl_data.rc_tx.queue.rd_idx].repetition_cnt = 0;

        /* trigger the timer interrupt to start waveforming */
        rc_tx_start_timer( rc_tx_prepare_transmission() );

        rc_tx_inc_rd_idx();
    }
}

/************************************************************************/
/* copyright: BBC-Hoerer 2016-2018                                      */
/************************************************************************/

