/******************************************************************************/
/* Project:   blind control set                                               */
/* Author:    BBC-Hoerer                                                      */
/*            You may use or modify this code for your purpose.               */
/*            BBC-Hoerer is not responsible for consequences using this code. */
/******************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "i2cmaster.h"
#include "lcd.h"
#include "_rl_com.h"

/* elemental functions of the real time clock rv3029 */

#define RV3029_ADDRESS    0xAC  /* fixed by chip - must not be changed */

/* setup the daylight saving time flag due to the date - called on startup, if a new date/time is set via hmi or if the date/time of summer/winter time switch has occured */
static void rv3029_determine_dst_flag  (void)
{
  if( (rl_data.rv3029.watch_el[RV3029_WATCH_EL_MONTH] >= 4) && (rl_data.rv3029.watch_el[RV3029_WATCH_EL_MONTH] < 11) )
  {
    rl_data.rv3029.dst_flag = 1;
  }
  else
  {
    rl_data.rv3029.dst_flag = 0;
  }
}


/* store booting time in eeprom */
void rv3029_store_boottime(uint8_t reset_flag)
{
    if(nv_data.btime.index>2)
    {
      nv_data.btime.index = 0;
    }
    else
    {
      nv_data.btime.index++;
    }

    nv_data.btime.reboot_ctr++;

    nv_data.btime.entry[nv_data.btime.index].watch_el[0] = rl_data.rv3029.watch_el[RV3029_WATCH_EL_DAY];
    nv_data.btime.entry[nv_data.btime.index].watch_el[1] = rl_data.rv3029.watch_el[RV3029_WATCH_EL_MONTH];
    nv_data.btime.entry[nv_data.btime.index].watch_el[2] = rl_data.rv3029.watch_el[RV3029_WATCH_EL_HOUR];
    nv_data.btime.entry[nv_data.btime.index].watch_el[3] = rl_data.rv3029.watch_el[RV3029_WATCH_EL_MINUTE];
    nv_data.btime.entry[nv_data.btime.index].reset_flag  = reset_flag;
    nv_data.btime.entry[nv_data.btime.index].line        = rl_noinit_data.line;

    eeprom_write_var_to_eeprom(EEPROM_ADDRESS(btime), (uint8_t *) &nv_data.btime, sizeof(NV_STRUC_BOOTTIME));
}


/* init function of the rv3029 */
void rv3029_init	(uint8_t reset_flag)
{
// Setup RTC chip.
	i2c_start_wait(RV3029_ADDRESS + I2C_WRITE);
	i2c_write(RV3029_REG_CON_1);  // CONTROL_1 address.
	i2c_write(0x19);              // CONTROL_1: Enable 1 Hz clock source an enable EEPROM refresh, enable self recovery
	i2c_write(0x00);              // CONTROL_INT: Disable all interrupts.
	i2c_write(0x00);              // CONTROL_INT_FLAG: Clear all.
	i2c_write(0x00);              // CONTROL_STATUS: Clear all, including PON.
  i2c_write(0x00);              // CONTROL_Reset.
	i2c_stop();

  rv3029_read_time();

  rv3029_determine_dst_flag();

  rv3029_store_boottime(reset_flag);

#if defined RV3029_SET_EEPROM
  // trickle charger
  i2c_start_wait(RV3029_ADDRESS + I2C_WRITE);
  i2c_write(RV3029_REG_EEPCTL);  // eeprom control register
  i2c_write(0x43);               // 20 kOhm, enable thermometer, scanning intervall 16s
  i2c_stop();
#endif
}

/* helping function for converting bcd to binary */
uint8_t rv3029_bcd2bin(uint8_t val)
{
	return ((val) & 0x0f) + ((val) >> 4) * 10;
}

/* helping function for converting binary to bcd */
uint8_t rv3029_bin2bcd(uint8_t val)
{
	return (((val / 10) << 4) | (val % 10));
}

/* set new date/time */
void rv3029_set_time(uint8_t * dt_field)
{
    rl_data.time.current_second = dt_field[RV3029_WATCH_EL_SECOND];
    rl_data.alm.trigger = 61-rl_data.time.current_second;

    i2c_start_wait(RV3029_ADDRESS + I2C_WRITE);
    i2c_write(RV3029_REG_CLK_SECONDS);                              // Address of second register.
    i2c_write(rv3029_bin2bcd(dt_field[RV3029_WATCH_EL_SECOND]));  // Set seconds
    i2c_write(rv3029_bin2bcd(dt_field[RV3029_WATCH_EL_MINUTE]));  // Set minutes
    i2c_write(rv3029_bin2bcd(dt_field[RV3029_WATCH_EL_HOUR]));    // Set hours
    i2c_write(rv3029_bin2bcd(dt_field[RV3029_WATCH_EL_DAY]));     // Set day
    i2c_write(rv3029_bin2bcd(dt_field[RV3029_WATCH_EL_WEEKDAY])); // Set weeek day
    i2c_write(rv3029_bin2bcd(dt_field[RV3029_WATCH_EL_MONTH]));   // Set month.
    i2c_write(rv3029_bin2bcd(dt_field[RV3029_WATCH_EL_YEAR]));    // Set year.
    i2c_stop();

    rv3029_read_time(); /* refresh the rl_data.rv3029.watch_el field */
    rv3029_determine_dst_flag();
}

/* read current date/time */
void rv3029_read_time(void)
{
    i2c_start_wait(RV3029_ADDRESS + I2C_WRITE);
    i2c_write(RV3029_REG_CLK_SECONDS);  				// Address of first time register (seconds).
    i2c_stop();
    i2c_start_wait(RV3029_ADDRESS + I2C_READ);
    rl_data.time.current_second = rl_data.rv3029.watch_el[RV3029_WATCH_EL_SECOND]  = rv3029_bcd2bin(i2c_readAck())& 0x7f;
    rl_data.rv3029.watch_el[RV3029_WATCH_EL_MINUTE]  = rv3029_bcd2bin(i2c_readAck())& 0x7f;
    rl_data.rv3029.watch_el[RV3029_WATCH_EL_HOUR]    = rv3029_bcd2bin(i2c_readAck())& 0x3f;
    rl_data.rv3029.watch_el[RV3029_WATCH_EL_DAY]     = rv3029_bcd2bin(i2c_readAck())& 0x3f;
    rl_data.rv3029.watch_el[RV3029_WATCH_EL_WEEKDAY] = rv3029_bcd2bin(i2c_readAck())& 0x07;
    rl_data.rv3029.watch_el[RV3029_WATCH_EL_MONTH]   = rv3029_bcd2bin(i2c_readAck())& 0x1f;
    rl_data.rv3029.watch_el[RV3029_WATCH_EL_YEAR]    = rv3029_bcd2bin(i2c_readNak())& 0x7f;
    i2c_stop();
}

#if 0
/* read and convert temperature measured by rv3029 */
void rv3029_write_temperature (uint8_t item)
{
    uint8_t temperature_value, temperature_sign;
    i2c_start_wait(RV3029_ADDRESS + I2C_WRITE);
    i2c_write(RV3029_REG_TEMP);          // Address of temp register
    i2c_stop();
    i2c_start_wait(RV3029_ADDRESS + I2C_READ);
    temperature_value = i2c_readNak();
    i2c_stop();

    if(temperature_value>=60)
    {
      temperature_value = temperature_value - 60;
      temperature_sign = '+';
    }
    else
    {
      temperature_value = 60 - temperature_value;
      temperature_sign = '-';
    }

    uint8_t x         = rl_data.menu.item_stack[item].x;
    uint8_t y         = rl_data.menu.item_stack[item].y;

    lcd_putc(x, y, temperature_sign);
    lcd_put_two_digit_dec_u8(x+1, y, temperature_value);
    lcd_puts(x+3, y, "\337C");
}
#endif

/* set only the hour of rv3029 */
void rv3029_set_hour(uint8_t correction)
{
    rl_data.rv3029.watch_el[RV3029_WATCH_EL_HOUR] = rl_data.rv3029.watch_el[RV3029_WATCH_EL_HOUR]+correction;

    i2c_start_wait(RV3029_ADDRESS + I2C_WRITE);
    i2c_write(RV3029_REG_CLK_HOURS);                              // Address of hour register.
    i2c_write(rv3029_bin2bcd(rl_data.rv3029.watch_el[RV3029_WATCH_EL_HOUR]));    // Set hours
    i2c_stop();

    rv3029_determine_dst_flag();
}


/* this function delivers the current hour, minute with a plus of 2 minutes */
/* the function is used to overtake the current time as start/end time for night or dayblind */
void rv3029_get_current_hour_minute (uint8_t * hour, uint8_t * minute)
{
  uint8_t curr_hour, curr_minute;

  curr_hour   = rl_data.rv3029.watch_el[RV3029_WATCH_EL_HOUR];
  curr_minute = rl_data.rv3029.watch_el[RV3029_WATCH_EL_MINUTE];

  if(curr_minute<58)
  {
    curr_minute += 2;
  }
  else
  {
    if(++curr_hour == 24 ) curr_hour = 0;

    curr_minute = (curr_minute==58) ? 0 : 1;
  }

  *hour   = curr_hour;
  *minute = curr_minute;
}


/************************************************************************/
/* copyright: BBC-Hoerer 2016-2018                                      */
/************************************************************************/

