/******************************************************************************/
/* Project:   blind control set                                               */
/* Author:    BBC-Hoerer                                                      */
/*            You may use or modify this code for your purpose.               */
/*            BBC-Hoerer is not responsible for consequences using this code. */
/******************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "i2cmaster.h"
#include "lcd.h"
#include "_rl_com.h"

/* wallbutton state machine */
/* short press button: alternating start/stop of the desired blind */
/* long press button:  run the blind motor as long as the wallbutton is pressed */
#define WB_STATE_HALT             0
#define WB_STATE_UPDRIVING_1      1
#define WB_STATE_UPDRIVING_2      2
#define WB_STATE_DOWNDRIVING_1    3
#define WB_STATE_DOWNDRIVING_2    4


/* init function of the wallbutton state machine */
void wb_init  (void)
{
  uint8_t rl_count;

  for(rl_count=0;rl_count<RL_COUNT;rl_count++)
  {
    rl_data.wb.state[rl_count] = WB_STATE_HALT;
  }
}


/* this function is called by the motor state machine if an associated motor has been halted to satisfy the wallbutton state machine */
void wb_enter_halt_state  (uint8_t rl_count)
{
  /* function is called after a motor_request with opcode MOTOR_OPC_HALT has been performed */
  rl_data.wb.state[rl_count] = WB_STATE_HALT;
}

/* call a motor request from the wallbutton state machine */
static void wb_motor_req  (uint8_t opc, uint8_t rl_count)
{
  MOTOR_REQ req;

  rl_data.alm.shadowing_sm[rl_count].state = ALM_M_AUTO_MODE_OFF; /* a manual request was done - so do not control this motor any more by d_blind automatic */

  req.opc = opc;

   /* ignore sensor when day blind shall drive up */
  req.rl_number   = rl_count;
  req.runtime     = MOTOR_RUNTIME_FULL;

  motor_request(&req);
}

/* evaluate the wallbutton state machine on any key change of up or down bench */
void wb_evaluate  (void)
{
    register KEY_BANK * up_keys   = (KEY_BANK *) & rl_data.key.bank[WB_UP_INDEX];
    register KEY_BANK * down_keys = (KEY_BANK *) & rl_data.key.bank[WB_DOWN_INDEX];

    if /* one of the 16 keys (WB_UP_INDEX/WB_DOWN_INDEX) was pressed/longpressed/released */
       ( up_keys->sum || down_keys->sum )
    {
      uint8_t rl_count, keybit;

      keybit = 0x01;

      for(rl_count=0;rl_count<RL_COUNT;rl_count++)
      {
        if( (up_keys->sum & keybit) || (down_keys->sum & keybit) )
        {
            rl_data.alm.n_blind_tmp_exclude_bitlane &= ~_BV(rl_count);

            if( (rl_data.key.service_mode.entered != 0) && (rl_data.key.service_mode.button == rl_count) )
            {
                /* skip a blind that is under service mode control */
                ;
            }
            else
            {
                  switch( rl_data.wb.state[rl_count] )
                  {
                    case WB_STATE_HALT:
                    {
                      if(up_keys->pressed & keybit)
                      {
                        wb_motor_req(MOTOR_OPC_UP, rl_count);

                        rl_data.wb.state[rl_count] = WB_STATE_UPDRIVING_1;
                      }
                      else if(down_keys->pressed & keybit)
                      {
                        wb_motor_req(MOTOR_OPC_DOWN, rl_count);

                        rl_data.wb.state[rl_count] = WB_STATE_DOWNDRIVING_1;
                      }
                      break;
                    }
                    case WB_STATE_UPDRIVING_1:
                    {
                      if(up_keys->pressed & keybit)
                      {
                        wb_motor_req(MOTOR_OPC_HALT, rl_count);
                      }
                      else if(up_keys->longpressed & keybit)
                      {
                        rl_data.wb.state[rl_count] = WB_STATE_UPDRIVING_2;
                      }
                      else if(down_keys->pressed & keybit)
                      {
                        wb_motor_req(MOTOR_OPC_DOWN, rl_count);

                        rl_data.wb.state[rl_count] = WB_STATE_DOWNDRIVING_1;
                      }
                      break;
                    }
                    case WB_STATE_UPDRIVING_2:
                    {
                      if(up_keys->released & keybit)
                      {
                        wb_motor_req(MOTOR_OPC_HALT, rl_count);
                      }
                      break;
                    }
                    case WB_STATE_DOWNDRIVING_1:
                    {
                      if(down_keys->pressed & keybit)
                      {
                        wb_motor_req(MOTOR_OPC_HALT, rl_count);
                      }
                      else if(down_keys->longpressed & keybit)
                      {
                        rl_data.wb.state[rl_count] = WB_STATE_DOWNDRIVING_2;
                      }
                      else if(up_keys->pressed & keybit)
                      {
                        wb_motor_req(MOTOR_OPC_UP, rl_count);

                        rl_data.wb.state[rl_count] = WB_STATE_UPDRIVING_1;
                      }
                      break;
                    }
                    case WB_STATE_DOWNDRIVING_2:
                    {
                      if(down_keys->released & keybit)
                      {
                        wb_motor_req(MOTOR_OPC_HALT, rl_count);
                      }
                      break;
                    }
                  }
             }
        }

        keybit = keybit<<1;
      }
    }
}

/************************************************************************/
/* copyright: BBC-Hoerer 2016-2018                                      */
/************************************************************************/

